@@ -31,6 +31,7 @@
 #include "winnls.h"
 #include "wine/list.h"
 #include "wine/rbtree.h"
+#include "wine/debug.h"
 
 #include "cvconst.h"
 
@@ -182,6 +183,8 @@ static inline BOOL symt_check_tag(const struct symt* s, enum SymTagEnum tag)
     return s && s->tag == tag;
 }
 
+typedef ULONG_PTR symref_t;
+
 /* lexical tree */
 struct symt_block
 {
@@ -215,7 +218,7 @@ struct symt_data
     struct hash_table_elt       hash_elt;       /* if global symbol */
     enum DataKind               kind;
     struct symt*                container;
-    struct symt*                type;
+    symref_t                    type;
     union                                       /* depends on kind */
     {
         /* DataIs{Global, FileStatic, StaticLocal}:
@@ -292,7 +295,7 @@ struct symt_function
     struct symt                 symt;           /* SymTagFunction or SymTagInlineSite */
     struct hash_table_elt       hash_elt;       /* if global symbol, inline site */
     struct symt*                container;      /* compiland (for SymTagFunction) or function (for SymTagInlineSite) */
-    struct symt*                type;           /* points to function_signature */
+    symref_t                    type;           /* points to function_signature */
     struct vector               vlines;
     struct vector               vchildren;      /* locals, params, blocks, start/end, labels, inline sites */
     struct symt_function*       next_inlinesite;/* linked list of inline sites in this function */
@@ -313,7 +316,7 @@ struct symt_public
     struct symt                 symt;
     struct hash_table_elt       hash_elt;
     struct symt*                container;      /* compiland */
-    BOOL is_function;
+    BOOL                        is_function;
     ULONG_PTR                   address;
     ULONG_PTR                   size;
 };
@@ -386,7 +389,7 @@ struct symt_typedef
 {
     struct symt                 symt;
     struct hash_table_elt       hash_elt;
-    struct symt*                type;
+    symref_t                    type;
 };
 
 struct symt_udt
@@ -414,14 +417,55 @@ enum format_info
     DFI_LAST
 };
 
-struct module_format
+struct lineinfo_t
 {
-    struct module*              module;
-    void                        (*remove)(struct process* pcs, struct module_format* modfmt);
-    void                        (*loc_compute)(struct process* pcs,
-                                               const struct module_format* modfmt,
+    BOOL                        unicode;
+    PVOID                       key;
+    DWORD                       line_number;
+    union
+    {
+        CHAR*                   file_nameA;
+        WCHAR*                  file_nameW;
+    };
+    DWORD64                     address;
+};
+
+struct module_format;
+enum method_result {MR_SUCCESS, MR_FAILURE, MR_NOT_FOUND};
+struct module_format_vtable
+{
+    /* module handling */
+    void                        (*remove)(struct module_format* modfmt);
+
+    /* index management */
+    enum method_result          (*request_symref_t)(struct module_format *modfmt, symref_t ref, IMAGEHLP_SYMBOL_TYPE_INFO req, void *data);
+
+    /* types management */
+    enum method_result          (*find_type)(struct module_format *modfmt, const char *name, symref_t *ref);
+    enum method_result          (*enumerate_types)(struct module_format *modfmt, BOOL (*cb)(symref_t, const char *, void*), void *user);
+
+    /* stack walk */
+    void                        (*loc_compute)(const struct module_format* modfmt,
                                                const struct symt_function* func,
                                                struct location* loc);
+    /* line information */
+    enum method_result          (*get_line_from_address)(struct module_format *modfmt,
+                                                         DWORD64 address, struct lineinfo_t *line_info);
+    enum method_result          (*advance_line_info)(struct module_format *modfmt,
+                                                     struct lineinfo_t *line_info, BOOL forward);
+    enum method_result          (*enumerate_lines)(struct module_format *modfmt, const WCHAR* compiland_regex,
+                                                   const WCHAR *source_file_regex, PSYM_ENUMLINES_CALLBACK cb, void *user);
+
+    /* source files information */
+    enum method_result          (*enumerate_sources)(struct module_format *modfmt, const WCHAR *sourcefile_regex,
+                                                     PSYM_ENUMSOURCEFILES_CALLBACKW cb, void *user);
+};
+
+struct module_format
+{
+    struct module*                      module;
+    const struct module_format_vtable*  vtable;
+
     union
     {
         struct elf_module_info*         elf_info;
@@ -453,6 +497,7 @@ struct module
     /* specific information for debug types */
     struct module_format*       format_info[DFI_LAST];
     unsigned                    debug_format_bitmask;
+    struct module_format       *ops_symref_modfmt; /* HACK for fast access to the ops table */
 
     /* memory allocation pool */
     struct pool                 pool;
@@ -478,6 +523,29 @@ struct module
     struct wine_rb_tree         sources_offsets_tree;
 };
 
+struct module_format_vtable_iterator
+{
+    int dfi;
+    struct module_format *modfmt;
+};
+
+#define MODULE_FORMAT_VTABLE_INDEX(f) (offsetof(struct module_format_vtable, f) / sizeof(void*))
+
+static inline BOOL module_format_vtable_iterator_next(struct module *module, struct module_format_vtable_iterator *iter, size_t method_index)
+{
+    for ( ; iter->dfi < DFI_LAST; iter->dfi++)
+    {
+        iter->modfmt = module->format_info[iter->dfi];
+        if (iter->modfmt && ((const void**)iter->modfmt->vtable)[method_index])
+        {
+            iter->dfi++;
+            return TRUE;
+        }
+    }
+    iter->modfmt = NULL;
+    return FALSE;
+}
+
 typedef BOOL (*enum_modules_cb)(const WCHAR*, ULONG_PTR addr, void* user);
 
 struct loader_ops
@@ -844,38 +912,38 @@ extern struct symt_public*
                                     ULONG_PTR address,
                                     unsigned size);
 extern struct symt_data*
-                    symt_new_global_variable(struct module* module, 
+                    symt_new_global_variable(struct module* module,
                                              struct symt_compiland* parent,
                                              const char* name, unsigned is_static,
                                              struct location loc, ULONG_PTR size,
-                                             struct symt* type);
+                                             symref_t type);
 extern struct symt_function*
                     symt_new_function(struct module* module,
                                       struct symt_compiland* parent,
                                       const char* name,
                                       ULONG_PTR addr, ULONG_PTR size,
-                                      struct symt* type);
+                                      symref_t type);
 extern struct symt_function*
                     symt_new_inlinesite(struct module* module,
                                         struct symt_function* func,
                                         struct symt* parent,
                                         const char* name,
-                                        struct symt* type,
+                                        symref_t type,
                                         unsigned num_ranges);
 extern void         symt_add_func_line(struct module* module,
                                        struct symt_function* func, 
                                        unsigned source_idx, int line_num, 
                                        ULONG_PTR offset);
 extern struct symt_data*
-                    symt_add_func_local(struct module* module, 
-                                        struct symt_function* func, 
+                    symt_add_func_local(struct module* module,
+                                        struct symt_function* func,
                                         enum DataKind dt, const struct location* loc,
                                         struct symt_block* block,
-                                        struct symt* type, const char* name);
+                                        symref_t, const char* name);
 extern struct symt_data*
                     symt_add_func_constant(struct module* module,
                                            struct symt_function* func, struct symt_block* block,
-                                           struct symt* type, const char* name, VARIANT* v);
+                                           symref_t, const char* name, VARIANT* v);
 extern struct symt_block*
                     symt_open_func_block(struct module* module,
                                          struct symt_function* func,
@@ -899,22 +967,33 @@ extern struct symt_thunk*
 extern struct symt_data*
                     symt_new_constant(struct module* module,
                                       struct symt_compiland* parent,
-                                      const char* name, struct symt* type,
+                                      const char* name, symref_t type,
                                       const VARIANT* v);
 extern struct symt_hierarchy_point*
                     symt_new_label(struct module* module,
                                    struct symt_compiland* compiland,
                                    const char* name, ULONG_PTR address);
-extern struct symt* symt_index2ptr(struct module* module, DWORD id);
-extern DWORD        symt_ptr2index(struct module* module, const struct symt* sym);
+static inline symref_t
+                    symt_ptr_to_symref(const struct symt *symt) {return (ULONG_PTR)symt;}
+extern symref_t     symt_index_to_symref(struct module* module, DWORD id);
+extern DWORD        symt_symref_to_index(struct module* module, symref_t sym);
+static inline DWORD symt_ptr_to_index(struct module *module, const struct symt *symt) {return symt_symref_to_index(module, symt_ptr_to_symref(symt));}
+static inline BOOL  symt_is_symref_ptr(symref_t ref) {return (ref & 3) == 0;}
 extern struct symt_custom*
                     symt_new_custom(struct module* module, const char* name,
                                     DWORD64 addr, DWORD size);
+extern BOOL         lineinfo_set_nameA(struct process* pcs, struct lineinfo_t* intl, char* str);
 
 /* type.c */
 extern void         symt_init_basic(struct module* module);
+extern BOOL         symt_get_info_raw(struct module* module, const struct symt* type,
+                                      IMAGEHLP_SYMBOL_TYPE_INFO req, void* pInfo);
 extern BOOL         symt_get_info(struct module* module, const struct symt* type,
                                   IMAGEHLP_SYMBOL_TYPE_INFO req, void* pInfo);
+extern BOOL         symt_get_info_from_index(struct module* module, DWORD index,
+                                             IMAGEHLP_SYMBOL_TYPE_INFO req, void* pInfo);
+extern BOOL         symt_get_info_from_symref(struct module* module, symref_t type,
+                                              IMAGEHLP_SYMBOL_TYPE_INFO req, void* pInfo);
 extern struct symt_basic*
                     symt_get_basic(enum BasicType, unsigned size);
 extern struct symt_udt*
@@ -922,10 +1001,10 @@ extern struct symt_udt*
                                  unsigned size, enum UdtKind kind);
 extern BOOL         symt_set_udt_size(struct module* module,
                                       struct symt_udt* type, unsigned size);
-extern BOOL         symt_add_udt_element(struct module* module, 
-                                         struct symt_udt* udt_type, 
+extern BOOL         symt_add_udt_element(struct module* module,
+                                         struct symt_udt* udt_type,
                                          const char* name,
-                                         struct symt* elt_type, unsigned offset, 
+                                         symref_t elt_type, unsigned offset,
                                          unsigned bit_offset, unsigned bit_size);
 extern struct symt_enum*
                     symt_new_enum(struct module* module, const char* typename,
@@ -944,11 +1023,11 @@ extern BOOL         symt_add_function_signature_parameter(struct module* module,
                                                           struct symt_function_signature* sig,
                                                           struct symt* param);
 extern struct symt_pointer*
-                    symt_new_pointer(struct module* module, 
+                    symt_new_pointer(struct module* module,
                                      struct symt* ref_type,
                                      ULONG_PTR size);
 extern struct symt_typedef*
-                    symt_new_typedef(struct module* module, struct symt* ref, 
+                    symt_new_typedef(struct module* module, symref_t ref,
                                      const char* name);
 extern struct symt_function*
                     symt_find_lowest_inlined(struct symt_function* func, DWORD64 addr);
@@ -985,3 +1064,15 @@ extern struct symt_function*
 #define IFC_DEPTH_MASK   0x3FFFFFFF
 #define IFC_MODE(x)      ((x) & ~IFC_DEPTH_MASK)
 #define IFC_DEPTH(x)     ((x) & IFC_DEPTH_MASK)
+
+/* temporary helpers for PDB rewriting */
+struct _PDB_FPO_DATA;
+extern BOOL pdb_fpo_unwind_parse_cmd_string(struct cpu_stack_walk* csw, struct _PDB_FPO_DATA* fpoext,
+                                            const char* cmd, struct pdb_cmd_pair* cpair);
+extern BOOL pdb_old_virtual_unwind(struct cpu_stack_walk *csw, DWORD_PTR ip,
+                                   union ctx *context, struct pdb_cmd_pair *cpair);
+struct pdb_reader;
+extern BOOL pdb_hack_get_main_info(struct module_format *modfmt, struct pdb_reader **pdb, unsigned *fpoext_stream);
+extern void pdb_reader_dispose(struct pdb_reader *pdb);
+extern struct pdb_reader *pdb_hack_reader_init(struct module *module, HANDLE file, const IMAGE_SECTION_HEADER *sections, unsigned num_sections);
+extern BOOL cv_hack_ptr_to_symref(struct pdb_reader *pdb, unsigned typeno, symref_t *symref);
