@@ -1190,7 +1190,7 @@ static void macho_finish_stabs(struct module* module, struct hash_table* ht_symt
             if (ste->is_code)
             {
                 symt_new_function(module, ste->compiland, ste->ht_elt.name,
-                    ste->addr, 0, NULL);
+                                  ste->addr, 0, 0);
             }
             else
             {
@@ -1200,7 +1200,7 @@ static void macho_finish_stabs(struct module* module, struct hash_table* ht_symt
                 loc.reg = 0;
                 loc.offset = ste->addr;
                 symt_new_global_variable(module, ste->compiland, ste->ht_elt.name,
-                                         !ste->is_global, loc, 0, NULL);
+                                         !ste->is_global, loc, 0, 0);
             }
 
             ste->used = 1;
