@@ -1384,6 +1384,7 @@ BOOL WINAPI GetTextExtentExPointW( HDC hdc, const WCHAR *str, INT count, INT max
     BOOL ret = TRUE;
 
     if (!(dc_attr = get_dc_attr( hdc ))) return FALSE;
+    if (count < 0) return FALSE;
 
     bidi_flags = dc_attr->text_align & TA_RTLREADING ? WINE_GCPW_FORCE_RTL : WINE_GCPW_FORCE_LTR;
     bidi_flags |= WINE_GCPW_DISABLE_REORDERING;
@@ -1421,8 +1422,11 @@ BOOL WINAPI GetTextExtentExPointW( HDC hdc, const WCHAR *str, INT count, INT max
                 if (extents + cluster_extent > max_ext) break;
                 if (nfit) *nfit += cluster_size;
 
-                for (j = 0; j < cluster_size; ++j)
-                    dxs[i + j] = extents + cluster_extent / cluster_size;
+                if (dxs)
+                {
+                    for (j = 0; j < cluster_size; ++j)
+                        dxs[i + j] = extents + cluster_extent / cluster_size;
+                }
 
                 extents += cluster_extent;
             }
@@ -2597,43 +2601,6 @@ HANDLE WINAPI AddFontMemResourceEx( void *ptr, DWORD size, void *dv, DWORD *coun
 static const char dos_string[0x40] = "This is a TrueType resource file";
 static const char FONTRES[] = {'F','O','N','T','R','E','S',':'};
 
-#include <pshpack1.h>
-struct fontdir
-{
-    WORD   num_of_resources;
-    WORD   res_id;
-    WORD   dfVersion;
-    DWORD  dfSize;
-    CHAR   dfCopyright[60];
-    WORD   dfType;
-    WORD   dfPoints;
-    WORD   dfVertRes;
-    WORD   dfHorizRes;
-    WORD   dfAscent;
-    WORD   dfInternalLeading;
-    WORD   dfExternalLeading;
-    BYTE   dfItalic;
-    BYTE   dfUnderline;
-    BYTE   dfStrikeOut;
-    WORD   dfWeight;
-    BYTE   dfCharSet;
-    WORD   dfPixWidth;
-    WORD   dfPixHeight;
-    BYTE   dfPitchAndFamily;
-    WORD   dfAvgWidth;
-    WORD   dfMaxWidth;
-    BYTE   dfFirstChar;
-    BYTE   dfLastChar;
-    BYTE   dfDefaultChar;
-    BYTE   dfBreakChar;
-    WORD   dfWidthBytes;
-    DWORD  dfDevice;
-    DWORD  dfFace;
-    DWORD  dfReserved;
-    CHAR   szFaceName[LF_FACESIZE];
-};
-#include <poppack.h>
-
 #include <pshpack2.h>
 
 struct ne_typeinfo
@@ -2673,6 +2640,7 @@ static BOOL create_fot( const WCHAR *resource, const WCHAR *font_file, const str
     BYTE *ptr, *start;
     BYTE import_name_len, res_name_len, non_res_name_len, font_file_len;
     char *font_fileA, *last_part, *ext;
+    const char *face_name;
     IMAGE_DOS_HEADER dos;
     IMAGE_OS2_HEADER ne =
     {
@@ -2712,7 +2680,8 @@ static BOOL create_fot( const WCHAR *resource, const WCHAR *font_file, const str
     if (ext) res_name_len = ext - last_part;
     else res_name_len = import_name_len - 1;
 
-    non_res_name_len = sizeof( FONTRES ) + strlen( fontdir->szFaceName );
+    face_name = fontdir->szFaceName + strlen( fontdir->szFaceName ) + 1;  /* skip family name */
+    non_res_name_len = sizeof( FONTRES ) + strlen( face_name );
 
     ne.ne_cbnrestab = 1 + non_res_name_len + 2 + 1; /* len + string + (WORD) ord_num + 1 byte eod */
     ne.ne_restab = ne.ne_rsrctab + sizeof(rsrc_tab);
@@ -2724,7 +2693,7 @@ static BOOL create_fot( const WCHAR *resource, const WCHAR *font_file, const str
     rsrc_tab.scalable_name.off = (ne.ne_nrestab + ne.ne_cbnrestab + 0xf) >> 4;
     rsrc_tab.scalable_name.len = (font_file_len + 0xf) >> 4;
     rsrc_tab.fontdir_name.off  = rsrc_tab.scalable_name.off + rsrc_tab.scalable_name.len;
-    rsrc_tab.fontdir_name.len  = (fontdir->dfSize + 0xf) >> 4;
+    rsrc_tab.fontdir_name.len  = (sizeof(*fontdir) + 0xf) >> 4;
 
     size = (rsrc_tab.fontdir_name.off + rsrc_tab.fontdir_name.len) << 4;
     start = ptr = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, size );
@@ -2753,13 +2722,13 @@ static BOOL create_fot( const WCHAR *resource, const WCHAR *font_file, const str
     ptr = start + ne.ne_nrestab;
     *ptr++ = non_res_name_len;
     memcpy( ptr, FONTRES, sizeof(FONTRES) );
-    memcpy( ptr + sizeof(FONTRES), fontdir->szFaceName, strlen( fontdir->szFaceName ) );
+    memcpy( ptr + sizeof(FONTRES), face_name, strlen( face_name ));
 
     ptr = start + (rsrc_tab.scalable_name.off << 4);
     memcpy( ptr, font_fileA, font_file_len );
 
     ptr = start + (rsrc_tab.fontdir_name.off << 4);
-    memcpy( ptr, fontdir, fontdir->dfSize );
+    memcpy( ptr, fontdir, sizeof(*fontdir) );
 
     file = CreateFileW( resource, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL );
     if (file != INVALID_HANDLE_VALUE)
@@ -2781,12 +2750,10 @@ static BOOL create_fot( const WCHAR *resource, const WCHAR *font_file, const str
 BOOL WINAPI CreateScalableFontResourceW( DWORD hidden, const WCHAR *resource_file,
                                          const WCHAR *font_file, const WCHAR *font_path )
 {
-    WCHAR path[MAX_PATH], face_name[128];
-    struct fontdir fontdir = { 0 };
+    WCHAR path[MAX_PATH];
+    struct fontdir fontdir;
     UNICODE_STRING nt_name;
-    TEXTMETRICW otm;
-    UINT em_square;
-    BOOL ret;
+    ULONG ret;
 
     TRACE("(%ld, %s, %s, %s)\n", hidden, debugstr_w(resource_file),
           debugstr_w(font_file), debugstr_w(font_path) );
@@ -2802,45 +2769,11 @@ BOOL WINAPI CreateScalableFontResourceW( DWORD hidden, const WCHAR *resource_fil
         if (!RtlDosPathNameToNtPathName_U( path, &nt_name, NULL, NULL )) goto done;
     }
     else if (!RtlDosPathNameToNtPathName_U( font_file, &nt_name, NULL, NULL )) goto done;
-    ret = __wine_get_file_outline_text_metric( nt_name.Buffer, &otm, &em_square, face_name );
+
+    ret = NtGdiMakeFontDir( hidden, (BYTE *)&fontdir, sizeof(fontdir),
+                            nt_name.Buffer, nt_name.Length + sizeof(WCHAR) );
     RtlFreeUnicodeString( &nt_name );
-    if (!ret) goto done;
-    if (!(otm.tmPitchAndFamily & TMPF_TRUETYPE)) goto done;
-
-    fontdir.num_of_resources  = 1;
-    fontdir.res_id            = 0;
-    fontdir.dfVersion         = 0x200;
-    fontdir.dfSize            = sizeof(fontdir);
-    strcpy( fontdir.dfCopyright, "Wine fontdir" );
-    fontdir.dfType            = 0x4003;  /* 0x0080 set if private */
-    fontdir.dfPoints          = em_square;
-    fontdir.dfVertRes         = 72;
-    fontdir.dfHorizRes        = 72;
-    fontdir.dfAscent          = otm.tmAscent;
-    fontdir.dfInternalLeading = otm.tmInternalLeading;
-    fontdir.dfExternalLeading = otm.tmExternalLeading;
-    fontdir.dfItalic          = otm.tmItalic;
-    fontdir.dfUnderline       = otm.tmUnderlined;
-    fontdir.dfStrikeOut       = otm.tmStruckOut;
-    fontdir.dfWeight          = otm.tmWeight;
-    fontdir.dfCharSet         = otm.tmCharSet;
-    fontdir.dfPixWidth        = 0;
-    fontdir.dfPixHeight       = otm.tmHeight;
-    fontdir.dfPitchAndFamily  = otm.tmPitchAndFamily;
-    fontdir.dfAvgWidth        = otm.tmAveCharWidth;
-    fontdir.dfMaxWidth        = otm.tmMaxCharWidth;
-    fontdir.dfFirstChar       = otm.tmFirstChar;
-    fontdir.dfLastChar        = otm.tmLastChar;
-    fontdir.dfDefaultChar     = otm.tmDefaultChar;
-    fontdir.dfBreakChar       = otm.tmBreakChar;
-    fontdir.dfWidthBytes      = 0;
-    fontdir.dfDevice          = 0;
-    fontdir.dfFace            = FIELD_OFFSET( struct fontdir, szFaceName );
-    fontdir.dfReserved        = 0;
-    WideCharToMultiByte( CP_ACP, 0, face_name, -1, fontdir.szFaceName, LF_FACESIZE, NULL, NULL );
-
-    if (hidden) fontdir.dfType |= 0x80;
-    return create_fot( resource_file, font_file, &fontdir );
+    if (ret) return create_fot( resource_file, font_file, &fontdir );
 
 done:
     SetLastError( ERROR_INVALID_PARAMETER );
