@@ -68,8 +68,14 @@ typedef struct {
     const WCHAR *name;
     UINT32 id;
     UINT32 iid;
+    UINT32 flags;
 } HostFunction;
 
+typedef struct {
+    FunctionInstance function;
+    IWineJSDispatchHost *host_iface;
+} HostConstructor;
+
 typedef struct {
     jsdisp_t jsdisp;
     jsval_t *buf;
@@ -195,7 +201,51 @@ static HRESULT Arguments_gc_traverse(struct gc_ctx *gc_ctx, enum gc_traverse_op
     return S_OK;
 }
 
+static HRESULT Arguments_get_caller(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
+{
+    ArgumentsInstance *arguments = arguments_from_jsdisp(jsthis);
+    call_frame_t *frame;
+    HRESULT hres;
+
+    TRACE("\n");
+
+    for(frame = ctx->call_ctx; frame; frame = frame->prev_frame) {
+        if(frame->arguments_obj == &arguments->jsdisp) {
+            frame = frame->prev_frame;
+            if(!frame || !frame->function_instance)
+                break;
+            if(!frame->arguments_obj) {
+                hres = setup_arguments_object(ctx, frame);
+                if(FAILED(hres))
+                    return hres;
+            }
+            *r = jsval_obj(jsdisp_addref(frame->arguments_obj));
+            return S_OK;
+        }
+    }
+
+    *r = jsval_null();
+    return S_OK;
+}
+
+static const builtin_prop_t Arguments_props[] = {
+    {L"caller",              NULL, 0,                        Arguments_get_caller},
+};
+
 static const builtin_info_t Arguments_info = {
+    .class       = JSCLASS_ARGUMENTS,
+    .call        = Arguments_value,
+    .props_cnt   = ARRAY_SIZE(Arguments_props),
+    .props       = Arguments_props,
+    .destructor  = Arguments_destructor,
+    .lookup_prop = Arguments_lookup_prop,
+    .next_prop   = Arguments_next_prop,
+    .prop_get    = Arguments_prop_get,
+    .prop_put    = Arguments_prop_put,
+    .gc_traverse = Arguments_gc_traverse
+};
+
+static const builtin_info_t Arguments_ES5_info = {
     .class       = JSCLASS_ARGUMENTS,
     .call        = Arguments_value,
     .destructor  = Arguments_destructor,
@@ -215,7 +265,7 @@ HRESULT setup_arguments_object(script_ctx_t *ctx, call_frame_t *frame)
     if(!args)
         return E_OUTOFMEMORY;
 
-    hres = init_dispex_from_constr(&args->jsdisp, ctx, &Arguments_info, ctx->object_constr);
+    hres = init_dispex_from_constr(&args->jsdisp, ctx, ctx->version < SCRIPTLANGUAGEVERSION_ES5 ? &Arguments_info : &Arguments_ES5_info, ctx->object_constr);
     if(FAILED(hres)) {
         free(args);
         return hres;
@@ -659,7 +709,7 @@ static const builtin_info_t FunctionInst_info = {
 };
 
 static HRESULT create_function(script_ctx_t *ctx, const builtin_info_t *builtin_info, const function_vtbl_t *vtbl, size_t size,
-        DWORD flags, BOOL funcprot, jsdisp_t *prototype, void **ret)
+        DWORD flags, jsdisp_t *prototype, void **ret)
 {
     FunctionInstance *function;
     HRESULT hres;
@@ -668,7 +718,7 @@ static HRESULT create_function(script_ctx_t *ctx, const builtin_info_t *builtin_
     if(!function)
         return E_OUTOFMEMORY;
 
-    if(funcprot)
+    if(prototype)
         hres = init_dispex(&function->dispex, ctx, builtin_info, prototype);
     else if(builtin_info)
         hres = init_dispex_from_constr(&function->dispex, ctx, builtin_info, ctx->function_constr);
@@ -729,7 +779,7 @@ HRESULT create_builtin_function(script_ctx_t *ctx, builtin_invoke_t value_proc,
     if(!ctx->function_constr)
         return E_UNEXPECTED;
 
-    hres = create_function(ctx, builtin_info, &NativeFunctionVtbl, sizeof(NativeFunction), flags, FALSE, NULL, (void**)&function);
+    hres = create_function(ctx, builtin_info, &NativeFunctionVtbl, sizeof(NativeFunction), flags, NULL, (void**)&function);
     if(FAILED(hres))
         return hres;
 
@@ -907,7 +957,7 @@ HRESULT create_source_function(script_ctx_t *ctx, bytecode_t *code, function_cod
     HRESULT hres;
 
     hres = create_function(ctx, &InterpretedFunction_info, &InterpretedFunctionVtbl, sizeof(InterpretedFunction),
-                           PROPF_CONSTR, FALSE, NULL, (void**)&function);
+                           PROPF_CONSTR, NULL, (void**)&function);
     if(FAILED(hres))
         return hres;
 
@@ -973,8 +1023,8 @@ static HRESULT HostFunction_call(script_ctx_t *ctx, FunctionInstance *func, jsva
 
     if(SUCCEEDED(hres)) {
         V_VT(&retv) = VT_EMPTY;
-        hres = IWineJSDispatchHost_CallFunction(obj, function->id, function->iid, &dp, r ? &retv : NULL, &ei,
-                                                &ctx->jscaller->IServiceProvider_iface);
+        hres = IWineJSDispatchHost_CallFunction(obj, function->id, function->iid, function->flags, &dp,
+                                                r ? &retv : NULL, &ei, &ctx->jscaller->IServiceProvider_iface);
         if(hres == DISP_E_EXCEPTION)
             handle_dispatch_exception(ctx, &ei);
         if(SUCCEEDED(hres) && r) {
@@ -1019,7 +1069,7 @@ static const function_vtbl_t HostFunctionVtbl = {
     HostFunction_gc_traverse
 };
 
-HRESULT create_host_function(script_ctx_t *ctx, const struct property_info *desc, jsdisp_t **ret)
+HRESULT create_host_function(script_ctx_t *ctx, const struct property_info *desc, DWORD flags, jsdisp_t **ret)
 {
     HostFunction *function;
     HRESULT hres;
@@ -1028,17 +1078,138 @@ HRESULT create_host_function(script_ctx_t *ctx, const struct property_info *desc
         return E_UNEXPECTED;
 
     hres = create_function(ctx, &HostFunction_info, &HostFunctionVtbl, sizeof(HostFunction), PROPF_METHOD,
-                           FALSE, NULL, (void**)&function);
+                           NULL, (void**)&function);
     if(FAILED(hres))
         return hres;
 
     function->name = desc->name;
     function->id = desc->id;
-    function->iid = desc->func_iid;
+    function->iid = desc->iid;
+    function->flags = flags;
     *ret = &function->function.dispex;
     return S_OK;
 }
 
+static ULONG HostConstructor_addref(jsdisp_t *jsdisp)
+{
+    HostConstructor *constr = (HostConstructor*)jsdisp;
+    return IWineJSDispatchHost_AddRef(constr->host_iface);
+}
+
+static ULONG HostConstructor_release(jsdisp_t *jsdisp)
+{
+    HostConstructor *constr = (HostConstructor*)jsdisp;
+    return IWineJSDispatchHost_Release(constr->host_iface);
+}
+
+static HRESULT HostConstructor_lookup_prop(jsdisp_t *jsdisp, const WCHAR *name, unsigned flags, struct property_info *desc)
+{
+    HostConstructor *constr = (HostConstructor*)jsdisp;
+    HRESULT hres = IWineJSDispatchHost_LookupProperty(constr->host_iface, name, flags, desc);
+    assert(hres != S_OK || (desc->flags & PROPF_METHOD)); /* external properties are not allowed */
+    return hres;
+}
+
+static const builtin_info_t HostConstructor_info = {
+    .class       = JSCLASS_FUNCTION,
+    .addref      = HostConstructor_addref,
+    .release     = HostConstructor_release,
+    .call        = Function_value,
+    .destructor  = Function_destructor,
+    .gc_traverse = Function_gc_traverse,
+    .lookup_prop = HostConstructor_lookup_prop,
+};
+
+static HRESULT HostConstructor_call(script_ctx_t *ctx, FunctionInstance *func, jsval_t vthis, unsigned flags,
+         unsigned argc, jsval_t *argv, jsval_t *r)
+{
+    HostConstructor *function = (HostConstructor*)func;
+    VARIANT buf[6], ret;
+    DISPPARAMS dp = { .cArgs = argc, .rgvarg = buf };
+    EXCEPINFO ei = { 0 };
+    HRESULT hres = S_OK;
+    unsigned i;
+
+    flags &= ~DISPATCH_JSCRIPT_INTERNAL_MASK;
+    if(argc > ARRAYSIZE(buf) && !(dp.rgvarg = malloc(argc * sizeof(*dp.rgvarg))))
+        return E_OUTOFMEMORY;
+
+    for(i = 0; i < argc; i++) {
+        hres = jsval_to_variant(argv[i], &dp.rgvarg[dp.cArgs - i - 1]);
+        if(FAILED(hres))
+            break;
+    }
+
+    if(SUCCEEDED(hres)) {
+        V_VT(&ret) = VT_EMPTY;
+        hres = IWineJSDispatchHost_Construct(function->host_iface, ctx->lcid, flags, &dp, &ret, &ei,
+                                             &ctx->jscaller->IServiceProvider_iface);
+        if(hres == DISP_E_EXCEPTION)
+            handle_dispatch_exception(ctx, &ei);
+        if(SUCCEEDED(hres)) {
+            if(r) hres = variant_to_jsval(ctx, &ret, r);
+            VariantClear(&ret);
+        }
+    }
+
+    while(i--)
+        VariantClear(&dp.rgvarg[dp.cArgs - i - 1]);
+    if(dp.rgvarg != buf)
+        free(dp.rgvarg);
+    return hres;
+}
+
+static HRESULT HostConstructor_toString(FunctionInstance *function, jsstr_t **ret)
+{
+    *ret = jsstr_alloc(L"\nfunction() {\n    [native code]\n}\n");
+    return *ret ? S_OK : E_OUTOFMEMORY;
+}
+
+static function_code_t *HostConstructor_get_code(FunctionInstance *function)
+{
+    return NULL;
+}
+
+static void HostConstructor_destructor(FunctionInstance *func)
+{
+}
+
+static HRESULT HostConstructor_gc_traverse(struct gc_ctx *gc_ctx, enum gc_traverse_op op, FunctionInstance *func)
+{
+    return S_OK;
+}
+
+static const function_vtbl_t HostConstructorVtbl = {
+    HostConstructor_call,
+    HostConstructor_toString,
+    HostConstructor_get_code,
+    HostConstructor_destructor,
+    HostConstructor_gc_traverse
+};
+
+HRESULT init_host_constructor(script_ctx_t *ctx, IWineJSDispatchHost *host_constr, IWineJSDispatch *prototype,
+                              IWineJSDispatch **ret)
+{
+    HostConstructor *function;
+    HRESULT hres;
+
+    hres = create_function(ctx, &HostConstructor_info, &HostConstructorVtbl, sizeof(*function), PROPF_METHOD,
+                           NULL, (void**)&function);
+    if(FAILED(hres))
+        return hres;
+    function->host_iface = host_constr;
+
+    hres = jsdisp_define_data_property(&function->function.dispex, L"prototype", PROPF_WRITABLE | PROPF_CONFIGURABLE,
+                                       jsval_disp((IDispatch *)prototype));
+    if(FAILED(hres)) {
+        IWineJSDispatch_Free(&function->function.dispex.IWineJSDispatch_iface);
+        return hres;
+    }
+
+    *ret = &function->function.dispex.IWineJSDispatch_iface;
+    return S_OK;
+}
+
 static HRESULT BindFunction_get_arguments(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
 {
     return JS_E_INVALID_ACTION;
@@ -1151,7 +1322,7 @@ static HRESULT create_bind_function(script_ctx_t *ctx, FunctionInstance *target,
     HRESULT hres;
 
     hres = create_function(ctx, &BindFunction_info, &BindFunctionVtbl, FIELD_OFFSET(BindFunction, args[argc]), PROPF_METHOD,
-                           FALSE, NULL, (void**)&function);
+                           NULL, (void**)&function);
     if(FAILED(hres))
         return hres;
 
@@ -1308,7 +1479,7 @@ HRESULT init_function_constr(script_ctx_t *ctx, jsdisp_t *object_prototype)
     HRESULT hres;
 
     hres = create_function(ctx, &Function_info, &NativeFunctionVtbl, sizeof(NativeFunction), PROPF_CONSTR,
-                           TRUE, object_prototype, (void**)&prot);
+                           object_prototype, (void**)&prot);
     if(FAILED(hres))
         return hres;
 
@@ -1316,7 +1487,7 @@ HRESULT init_function_constr(script_ctx_t *ctx, jsdisp_t *object_prototype)
     prot->name = L"prototype";
 
     hres = create_function(ctx, &FunctionInst_info, &NativeFunctionVtbl, sizeof(NativeFunction), PROPF_CONSTR|1,
-                           TRUE, &prot->function.dispex, (void**)&constr);
+                           &prot->function.dispex, (void**)&constr);
     if(SUCCEEDED(hres)) {
         constr->proc = FunctionConstr_value;
         constr->name = L"Function";
