@@ -2022,6 +2022,8 @@ static void test_debugger(const char *argv0)
                "ExceptionAddress = %p\n", ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress);
             exception_cnt++;
             if (event_order == 1) event_order = 2; /* exception debug event after exit thread event */
+
+            if (ctx.ev.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_ACCESS_VIOLATION) break;
         }
 
         trace("received %u exceptions\n", exception_cnt);
@@ -2190,8 +2192,10 @@ static void test_debugger(const char *argv0)
     {
         next_event(&ctx, WAIT_EVENT_TIMEOUT);
         ok (ctx.ev.dwDebugEventCode != EXCEPTION_DEBUG_EVENT, "got exception\n");
+        if (ctx.ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT) break;
     }
     while (ctx.ev.dwDebugEventCode != EXIT_PROCESS_DEBUG_EVENT);
+    if (ctx.ev.dwDebugEventCode != EXIT_PROCESS_DEBUG_EVENT) TerminateProcess(pi.hProcess, 0);
 
     ret = CloseHandle(event);
     ok(ret, "CloseHandle failed, last error %ld.\n", GetLastError());
