@@ -52,9 +52,10 @@ typedef struct {
 
 typedef struct {
     DISPID id;
-    BSTR name;
     tid_t tid;
+    BSTR name;
     dispex_hook_invoke_t hook;
+    BOOLEAN on_prototype;
     SHORT call_vtbl_off;
     SHORT put_vtbl_off;
     SHORT get_vtbl_off;
@@ -437,7 +438,8 @@ static void add_func_info(dispex_data_t *data, tid_t tid, const FUNCDESC *desc,
     }
 }
 
-static HRESULT process_interface(dispex_data_t *data, tid_t tid, ITypeInfo *disp_typeinfo, const dispex_hook_t *hooks)
+static HRESULT process_interface(dispex_data_t *data, tid_t tid, ITypeInfo *disp_typeinfo, const dispex_hook_t *hooks,
+                                 const DISPID *allow_list)
 {
     unsigned i = 7; /* skip IDispatch functions */
     ITypeInfo *typeinfo;
@@ -450,6 +452,7 @@ static HRESULT process_interface(dispex_data_t *data, tid_t tid, ITypeInfo *disp
 
     while(1) {
         const dispex_hook_t *hook = NULL;
+        const DISPID *dispid = NULL;
 
         hres = ITypeInfo_GetFuncDesc(typeinfo, i++, &funcdesc);
         if(FAILED(hres))
@@ -464,6 +467,17 @@ static HRESULT process_interface(dispex_data_t *data, tid_t tid, ITypeInfo *disp
                 hook = NULL;
         }
 
+        if(allow_list) {
+            for(dispid = allow_list; *dispid != DISPID_UNKNOWN; dispid++) {
+                if(*dispid == funcdesc->memid)
+                    break;
+            }
+            if(*dispid == DISPID_UNKNOWN) {
+                ITypeInfo_ReleaseFuncDesc(typeinfo, funcdesc);
+                continue;
+            }
+        }
+
         if(!hook || hook->invoke || hook->name) {
             add_func_info(data, tid, funcdesc, disp_typeinfo ? disp_typeinfo : typeinfo,
                           hook ? hook->invoke : NULL, hook ? hook->name : NULL);
@@ -479,7 +493,16 @@ void dispex_info_add_interface(dispex_data_t *info, tid_t tid, const dispex_hook
 {
     HRESULT hres;
 
-    hres = process_interface(info, tid, NULL, hooks);
+    hres = process_interface(info, tid, NULL, hooks, NULL);
+    if(FAILED(hres))
+        ERR("process_interface failed: %08lx\n", hres);
+}
+
+void dispex_info_add_dispids(dispex_data_t *info, tid_t tid, const DISPID *dispids)
+{
+    HRESULT hres;
+
+    hres = process_interface(info, tid, NULL, NULL, dispids);
     if(FAILED(hres))
         ERR("process_interface failed: %08lx\n", hres);
 }
@@ -523,7 +546,7 @@ static BOOL find_prototype_member(const dispex_data_t *info, DISPID id)
 
 static const char *object_names[] = {
 #define X(name) #name,
-    ALL_PROTOTYPES
+    ALL_OBJECTS
 #undef X
 };
 
@@ -573,7 +596,7 @@ static dispex_data_t *preprocess_dispex_data(dispex_static_data_t *desc, compat_
 
     if(desc->iface_tids) {
         for(tid = desc->iface_tids; *tid; tid++) {
-            hres = process_interface(data, *tid, dti, NULL);
+            hres = process_interface(data, *tid, dti, NULL, NULL);
             if(FAILED(hres))
                 break;
         }
@@ -594,8 +617,10 @@ static dispex_data_t *preprocess_dispex_data(dispex_static_data_t *desc, compat_
     data->name_table = malloc(data->func_cnt * sizeof(func_info_t*));
     for(i=0; i < data->func_cnt; i++) {
         /* Don't expose properties that are exposed by object's prototype */
-        if(find_prototype_member(data, data->funcs[i].id))
+        if(find_prototype_member(data, data->funcs[i].id)) {
+            data->funcs[i].on_prototype = TRUE;
             continue;
+        }
         data->name_table[data->name_cnt++] = data->funcs+i;
     }
     qsort(data->name_table, data->name_cnt, sizeof(func_info_t*), func_name_cmp);
@@ -1764,6 +1789,26 @@ HRESULT dispex_call_builtin(DispatchEx *dispex, DISPID id, DISPPARAMS *dp,
     return call_builtin_function(dispex, func, dp, res, ei, caller);
 }
 
+static VARIANT_BOOL reset_builtin_func(DispatchEx *dispex, func_info_t *func)
+{
+    func_obj_entry_t *entry;
+
+    if(!dispex->dynamic_data || !dispex->dynamic_data->func_disps ||
+       !dispex->dynamic_data->func_disps[func->func_disp_idx].func_obj)
+        return VARIANT_FALSE;
+
+    entry = dispex->dynamic_data->func_disps + func->func_disp_idx;
+    if(V_VT(&entry->val) == VT_DISPATCH &&
+       V_DISPATCH(&entry->val) == (IDispatch*)&entry->func_obj->dispex.IWineJSDispatchHost_iface)
+        return VARIANT_FALSE;
+
+    VariantClear(&entry->val);
+    V_VT(&entry->val) = VT_DISPATCH;
+    V_DISPATCH(&entry->val) = (IDispatch*)&entry->func_obj->dispex.IWineJSDispatchHost_iface;
+    IDispatch_AddRef(V_DISPATCH(&entry->val));
+    return VARIANT_TRUE;
+}
+
 HRESULT remove_attribute(DispatchEx *This, DISPID id, VARIANT_BOOL *success)
 {
     switch(get_dispid_type(id)) {
@@ -1793,26 +1838,7 @@ HRESULT remove_attribute(DispatchEx *This, DISPID id, VARIANT_BOOL *success)
 
         /* For builtin functions, we set their value to the original function. */
         if(func->func_disp_idx >= 0) {
-            func_obj_entry_t *entry;
-
-            if(!This->dynamic_data || !This->dynamic_data->func_disps
-                || !This->dynamic_data->func_disps[func->func_disp_idx].func_obj) {
-                *success = VARIANT_FALSE;
-                return S_OK;
-            }
-
-            entry = This->dynamic_data->func_disps + func->func_disp_idx;
-            if(V_VT(&entry->val) == VT_DISPATCH
-                    && V_DISPATCH(&entry->val) == (IDispatch*)&entry->func_obj->dispex.IWineJSDispatchHost_iface) {
-                *success = VARIANT_FALSE;
-                return S_OK;
-            }
-
-            VariantClear(&entry->val);
-            V_VT(&entry->val) = VT_DISPATCH;
-            V_DISPATCH(&entry->val) = (IDispatch*)&entry->func_obj->dispex.IWineJSDispatchHost_iface;
-            IDispatch_AddRef(V_DISPATCH(&entry->val));
-            *success = VARIANT_TRUE;
+            *success = reset_builtin_func(This, func);
             return S_OK;
         }
         *success = VARIANT_TRUE;
@@ -1835,32 +1861,6 @@ HRESULT remove_attribute(DispatchEx *This, DISPID id, VARIANT_BOOL *success)
     }
 }
 
-HRESULT dispex_to_string(DispatchEx *dispex, BSTR *ret)
-{
-    static const WCHAR prefix[8] = L"[object ";
-    static const WCHAR suffix[] = L"]";
-    WCHAR buf[ARRAY_SIZE(prefix) + ARRAY_SIZE(dispex->info->desc->prototype_name) + ARRAY_SIZE(suffix)], *p = buf;
-    compat_mode_t compat_mode = dispex_compat_mode(dispex);
-    const char *name = dispex->info->name;
-
-    if(!ret)
-        return E_INVALIDARG;
-
-    memcpy(p, prefix, sizeof(prefix));
-    p += ARRAY_SIZE(prefix);
-    if(compat_mode < COMPAT_MODE_IE9)
-        p--;
-    else {
-        while(*name)
-            *p++ = *name++;
-        assert(p + ARRAY_SIZE(suffix) - buf <= ARRAY_SIZE(buf));
-    }
-    memcpy(p, suffix, sizeof(suffix));
-
-    *ret = SysAllocString(buf);
-    return *ret ? S_OK : E_OUTOFMEMORY;
-}
-
 static dispex_data_t *ensure_dispex_info(dispex_static_data_t *desc, compat_mode_t compat_mode)
 {
     if(!desc->info_cache[compat_mode]) {
@@ -1905,22 +1905,23 @@ static BOOL ensure_real_info(DispatchEx *dispex)
     compat_mode_t compat_mode;
     HTMLInnerWindow *script_global;
     DispatchEx *prototype = NULL;
+    dispex_static_data_t *data;
 
     if(dispex->info != dispex->info->desc->delayed_init_info)
         return TRUE;
 
-    script_global = dispex->info->vtbl->get_script_global(dispex);
+    script_global = dispex->info->vtbl->get_script_global(dispex, &data);
     compat_mode = script_global->doc->document_mode;
 
-    if(compat_mode >= COMPAT_MODE_IE9 && dispex->info->desc->id) {
-        HRESULT hres = get_prototype(script_global, dispex->info->desc->id, &prototype);
+    if(compat_mode >= COMPAT_MODE_IE9 && data->id) {
+        HRESULT hres = get_prototype(script_global, data->id, &prototype);
         if(FAILED(hres)) {
             ERR("could not get prototype: %08lx\n", hres);
             return FALSE;
         }
     }
 
-    if (!(dispex->info = ensure_dispex_info(dispex->info->desc, compat_mode)))
+    if(!(dispex->info = ensure_dispex_info(data, compat_mode)))
         return FALSE;
     init_host_object(dispex, script_global, prototype);
     return TRUE;
@@ -1933,6 +1934,37 @@ compat_mode_t dispex_compat_mode(DispatchEx *dispex)
     return dispex->info->compat_mode;
 }
 
+HRESULT dispex_to_string(DispatchEx *dispex, BSTR *ret)
+{
+    static const WCHAR prefix[8] = L"[object ";
+    static const WCHAR suffix[] = L"]";
+    WCHAR buf[ARRAY_SIZE(prefix) + ARRAY_SIZE(dispex->info->desc->prototype_name) + ARRAY_SIZE(suffix)], *p = buf;
+    compat_mode_t compat_mode = dispex_compat_mode(dispex);
+    const char *name;
+
+    if(!ret)
+        return E_INVALIDARG;
+
+    memcpy(p, prefix, sizeof(prefix));
+    p += ARRAY_SIZE(prefix);
+    if(compat_mode < COMPAT_MODE_IE9)
+        p--;
+    else {
+        if(!ensure_real_info(dispex))
+            return E_OUTOFMEMORY;
+        name = dispex->info->name;
+        if(dispex->info->vtbl->get_name)
+            name = dispex->info->vtbl->get_name(dispex);
+        while(*name)
+            *p++ = *name++;
+        assert(p + ARRAY_SIZE(suffix) - buf <= ARRAY_SIZE(buf));
+    }
+    memcpy(p, suffix, sizeof(suffix));
+
+    *ret = SysAllocString(buf);
+    return *ret ? S_OK : E_OUTOFMEMORY;
+}
+
 static inline DispatchEx *impl_from_IWineJSDispatchHost(IWineJSDispatchHost *iface)
 {
     return CONTAINING_RECORD(iface, DispatchEx, IWineJSDispatchHost_iface);
@@ -2304,7 +2336,8 @@ static HRESULT dispex_prop_delete(DispatchEx *dispex, DISPID id)
         return E_NOTIMPL;
     }
 
-    if(is_dynamic_dispid(id)) {
+    switch(get_dispid_type(id)) {
+    case DISPEXPROP_DYNAMIC: {
         DWORD idx = id - DISPID_DYNPROP_0;
         dynamic_prop_t *prop;
 
@@ -2316,6 +2349,24 @@ static HRESULT dispex_prop_delete(DispatchEx *dispex, DISPID id)
         prop->flags |= DYNPROP_DELETED;
         return S_OK;
     }
+    case DISPEXPROP_BUILTIN: {
+        func_info_t *func;
+        HRESULT hres;
+
+        if(!ensure_real_info(dispex))
+            return E_OUTOFMEMORY;
+
+        hres = get_builtin_func(dispex->info, id, &func);
+        if(FAILED(hres))
+            return hres;
+
+        if(func->func_disp_idx >= 0)
+            reset_builtin_func(dispex, func);
+        return S_OK;
+    }
+    default:
+        break;
+    }
 
     return S_OK;
 }
@@ -2435,7 +2486,7 @@ static HRESULT next_dynamic_id(DispatchEx *dispex, DWORD idx, DISPID *ret_id)
     return S_OK;
 }
 
-HRESULT dispex_next_id(DispatchEx *dispex, DISPID id, DISPID *ret)
+HRESULT dispex_next_id(DispatchEx *dispex, DISPID id, BOOL enum_all_own_props, DISPID *ret)
 {
     func_info_t *func;
     HRESULT hres;
@@ -2460,7 +2511,7 @@ HRESULT dispex_next_id(DispatchEx *dispex, DISPID id, DISPID *ret)
         }
 
         while(func < dispex->info->funcs + dispex->info->func_cnt) {
-            if(func->func_disp_idx == -1) {
+            if(enum_all_own_props ? (!func->on_prototype) : (func->func_disp_idx == -1)) {
                 *ret = func->id;
                 return S_OK;
             }
@@ -2493,7 +2544,7 @@ static HRESULT WINAPI DispatchEx_GetNextDispID(IWineJSDispatchHost *iface, DWORD
         return E_OUTOFMEMORY;
     if(This->jsdisp)
         return IWineJSDispatch_GetNextDispID(This->jsdisp, grfdex, id, pid);
-    return dispex_next_id(This, id, pid);
+    return dispex_next_id(This, id, FALSE, pid);
 }
 
 static HRESULT WINAPI DispatchEx_GetNameSpaceParent(IWineJSDispatchHost *iface, IUnknown **ppunk)
@@ -2596,7 +2647,7 @@ static HRESULT WINAPI JSDispatchHost_NextProperty(IWineJSDispatchHost *iface, DI
 
     TRACE("%s (%p)->(%lx)\n", This->info->name, This, id);
 
-    hres = dispex_next_id(This, id, &next);
+    hres = dispex_next_id(This, id, TRUE, &next);
     if(hres != S_OK)
         return hres;
 
@@ -2918,7 +2969,7 @@ void init_dispatch_with_owner(DispatchEx *dispex, dispex_static_data_t *desc, Di
 dispex_static_data_t *object_descriptors[] = {
     NULL,
 #define X(name) &name ## _dispex,
-    ALL_PROTOTYPES
+    ALL_OBJECTS
 #undef X
 };
 
@@ -2956,7 +3007,7 @@ static const dispex_static_data_vtbl_t prototype_dispex_vtbl = {
     .find_dispid = prototype_find_dispid,
 };
 
-HRESULT get_prototype(HTMLInnerWindow *script_global, prototype_id_t id, DispatchEx **ret)
+HRESULT get_prototype(HTMLInnerWindow *script_global, object_id_t id, DispatchEx **ret)
 {
     compat_mode_t compat_mode = script_global->doc->document_mode;
     DispatchEx *prototype, *prot_prototype = NULL;
@@ -3004,7 +3055,7 @@ HRESULT get_prototype(HTMLInnerWindow *script_global, prototype_id_t id, Dispatc
 struct constructor
 {
     DispatchEx dispex;
-    prototype_id_t id;
+    object_id_t id;
 };
 
 static inline struct constructor *constr_from_DispatchEx(DispatchEx *iface)
@@ -3043,9 +3094,16 @@ static HRESULT constructor_find_dispid(DispatchEx *dispex, const WCHAR *name, DW
     return hres;
 }
 
+static const char *constructor_get_name(DispatchEx *dispex)
+{
+    struct constructor *constr = constr_from_DispatchEx(dispex);
+    return object_names[constr->id - 1];
+}
+
 static const dispex_static_data_vtbl_t constructor_dispex_vtbl = {
     .destructor  = constructor_destructor,
     .find_dispid = constructor_find_dispid,
+    .get_name    = constructor_get_name,
 };
 
 static dispex_static_data_t constructor_dispex = {
@@ -3054,12 +3112,10 @@ static dispex_static_data_t constructor_dispex = {
     .js_flags = HOSTOBJ_CONSTRUCTOR,
 };
 
-HRESULT get_constructor(HTMLInnerWindow *script_global, prototype_id_t id, DispatchEx **ret)
+HRESULT get_constructor(HTMLInnerWindow *script_global, object_id_t id, DispatchEx **ret)
 {
     dispex_static_data_t *info;
 
-    assert(script_global->doc->document_mode >= COMPAT_MODE_IE9);
-
     if(script_global->constructors[id]) {
         *ret = script_global->constructors[id];
         return S_OK;
@@ -3072,6 +3128,9 @@ HRESULT get_constructor(HTMLInnerWindow *script_global, prototype_id_t id, Dispa
             return hres;
     }else {
         struct constructor *constr;
+
+        assert(script_global->doc->document_mode >= COMPAT_MODE_IE9);
+
         if(!(constr = calloc(sizeof(*constr), 1)))
             return E_OUTOFMEMORY;
 
