@@ -341,16 +341,16 @@ static const event_target_vtbl_t HTMLOptionElement_event_target_vtbl = {
 };
 
 static const tid_t HTMLOptionElement_iface_tids[] = {
-    HTMLELEMENT_TIDS,
     IHTMLOptionElement_tid,
     0
 };
-static dispex_static_data_t HTMLOptionElement_dispex = {
-    "HTMLOptionElement",
-    &HTMLOptionElement_event_target_vtbl.dispex_vtbl,
-    DispHTMLOptionElement_tid,
-    HTMLOptionElement_iface_tids,
-    HTMLElement_init_dispex_info
+dispex_static_data_t HTMLOptionElement_dispex = {
+    .id           = OBJID_HTMLOptionElement,
+    .prototype_id = OBJID_HTMLElement,
+    .vtbl         = &HTMLOptionElement_event_target_vtbl.dispex_vtbl,
+    .disp_tid     = DispHTMLOptionElement_tid,
+    .iface_tids   = HTMLOptionElement_iface_tids,
+    .init_info    = HTMLElement_init_dispex_info,
 };
 
 HRESULT HTMLOptionElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
@@ -510,10 +510,11 @@ static HRESULT HTMLOptionElementFactory_value(DispatchEx *dispex, LCID lcid,
     return S_OK;
 }
 
-static const tid_t HTMLOptionElementFactory_iface_tids[] = {
-    IHTMLOptionElementFactory_tid,
-    0
-};
+static void HTMLOptionElementFactory_init_dispex_info(dispex_data_t *info, compat_mode_t mode)
+{
+    if(mode < COMPAT_MODE_IE9)
+        dispex_info_add_interface(info, IHTMLOptionElementFactory_tid, NULL);
+}
 
 static const dispex_static_data_vtbl_t HTMLOptionElementFactory_dispex_vtbl = {
     .query_interface  = HTMLOptionElementFactory_query_interface,
@@ -523,14 +524,7 @@ static const dispex_static_data_vtbl_t HTMLOptionElementFactory_dispex_vtbl = {
     .value            = HTMLOptionElementFactory_value,
 };
 
-static dispex_static_data_t HTMLOptionElementFactory_dispex = {
-    "Function",
-    &HTMLOptionElementFactory_dispex_vtbl,
-    IHTMLOptionElementFactory_tid,
-    HTMLOptionElementFactory_iface_tids,
-};
-
-HRESULT HTMLOptionElementFactory_Create(HTMLInnerWindow *window, HTMLOptionElementFactory **ret_ptr)
+static HRESULT HTMLOptionElementFactory_Create(HTMLInnerWindow *window, DispatchEx **ret_ptr)
 {
     HTMLOptionElementFactory *ret;
 
@@ -542,12 +536,22 @@ HRESULT HTMLOptionElementFactory_Create(HTMLInnerWindow *window, HTMLOptionEleme
     ret->window = window;
     IHTMLWindow2_AddRef(&window->base.IHTMLWindow2_iface);
 
-    init_dispatch(&ret->dispex, &HTMLOptionElementFactory_dispex, dispex_compat_mode(&window->event_target.dispex));
+    init_dispatch(&ret->dispex, &Option_dispex, window,
+                  dispex_compat_mode(&window->event_target.dispex));
 
-    *ret_ptr = ret;
+    *ret_ptr = &ret->dispex;
     return S_OK;
 }
 
+dispex_static_data_t Option_dispex = {
+    .name             = "Function",
+    .constructor_id   = OBJID_HTMLOptionElement,
+    .init_constructor = HTMLOptionElementFactory_Create,
+    .vtbl             = &HTMLOptionElementFactory_dispex_vtbl,
+    .disp_tid         = IHTMLOptionElementFactory_tid,
+    .init_info        = HTMLOptionElementFactory_init_dispex_info,
+};
+
 struct HTMLSelectElement {
     HTMLElement element;
 
@@ -1214,7 +1218,7 @@ static void HTMLSelectElement_unlink(DispatchEx *dispex)
 
 #define DISPID_OPTIONCOL_0 MSHTML_DISPID_CUSTOM_MIN
 
-static HRESULT HTMLSelectElement_get_dispid(DispatchEx *dispex, BSTR name, DWORD flags, DISPID *dispid)
+static HRESULT HTMLSelectElement_get_dispid(DispatchEx *dispex, const WCHAR *name, DWORD flags, DISPID *dispid)
 {
     const WCHAR *ptr;
     DWORD idx = 0;
@@ -1233,19 +1237,6 @@ static HRESULT HTMLSelectElement_get_dispid(DispatchEx *dispex, BSTR name, DWORD
     return S_OK;
 }
 
-static HRESULT HTMLSelectElement_dispex_get_name(DispatchEx *dispex, DISPID id, BSTR *name)
-{
-    DWORD idx = id - DISPID_OPTIONCOL_0;
-    WCHAR buf[11];
-    UINT len;
-
-    if(idx > MSHTML_CUSTOM_DISPID_CNT)
-        return DISP_E_MEMBERNOTFOUND;
-
-    len = swprintf(buf, ARRAY_SIZE(buf), L"%u", idx);
-    return (*name = SysAllocStringLen(buf, len)) ? S_OK : E_OUTOFMEMORY;
-}
-
 static HRESULT HTMLSelectElement_invoke(DispatchEx *dispex, DISPID id, LCID lcid, WORD flags, DISPPARAMS *params,
         VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
 {
@@ -1296,7 +1287,7 @@ static const event_target_vtbl_t HTMLSelectElement_event_target_vtbl = {
         .traverse       = HTMLSelectElement_traverse,
         .unlink         = HTMLSelectElement_unlink,
         .get_dispid     = HTMLSelectElement_get_dispid,
-        .get_name       = HTMLSelectElement_dispex_get_name,
+        .get_prop_desc  = dispex_index_prop_desc,
         .invoke         = HTMLSelectElement_invoke
     },
     HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
@@ -1304,17 +1295,17 @@ static const event_target_vtbl_t HTMLSelectElement_event_target_vtbl = {
 };
 
 static const tid_t HTMLSelectElement_tids[] = {
-    HTMLELEMENT_TIDS,
     IHTMLSelectElement_tid,
     0
 };
 
-static dispex_static_data_t HTMLSelectElement_dispex = {
-    "HTMLSelectElement",
-    &HTMLSelectElement_event_target_vtbl.dispex_vtbl,
-    DispHTMLSelectElement_tid,
-    HTMLSelectElement_tids,
-    HTMLElement_init_dispex_info
+dispex_static_data_t HTMLSelectElement_dispex = {
+    .id           = OBJID_HTMLSelectElement,
+    .prototype_id = OBJID_HTMLElement,
+    .vtbl         = &HTMLSelectElement_event_target_vtbl.dispex_vtbl,
+    .disp_tid     = DispHTMLSelectElement_tid,
+    .iface_tids   = HTMLSelectElement_tids,
+    .init_info    = HTMLElement_init_dispex_info,
 };
 
 HRESULT HTMLSelectElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
