@@ -1212,7 +1212,7 @@ static const tid_t mutation_observer_iface_tids[] = {
     0
 };
 dispex_static_data_t MutationObserver_dispex = {
-    .id               = PROT_MutationObserver,
+    .id               = OBJID_MutationObserver,
     .init_constructor = create_mutation_observer_ctor,
     .vtbl             = &mutation_observer_dispex_vtbl,
     .disp_tid         = IWineMSHTMLMutationObserver_tid,
@@ -1311,7 +1311,7 @@ static const dispex_static_data_vtbl_t mutation_observer_ctor_dispex_vtbl = {
 
 static dispex_static_data_t mutation_observer_ctor_dispex = {
     .name           = "Function",
-    .constructor_id = PROT_MutationObserver,
+    .constructor_id = OBJID_MutationObserver,
     .vtbl           = &mutation_observer_ctor_dispex_vtbl,
 };
 
