@@ -169,12 +169,7 @@ static DWORD WINAPI init_debug_lists( RTL_RUN_ONCE *once, void *param, void **co
     HANDLE root, hkey;
 
     RtlOpenCurrentUser( KEY_ALL_ACCESS, &root );
-    attr.Length = sizeof(attr);
-    attr.RootDirectory = root;
-    attr.ObjectName = &name;
-    attr.Attributes = 0;
-    attr.SecurityDescriptor = NULL;
-    attr.SecurityQualityOfService = NULL;
+    InitializeObjectAttributes( &attr, &name, OBJ_CASE_INSENSITIVE, root, NULL );
 
     /* @@ Wine registry key: HKCU\Software\Wine\Debug */
     if (NtOpenKey( &hkey, KEY_ALL_ACCESS, &attr )) hkey = 0;
@@ -454,10 +449,8 @@ void * WINAPI relay_trace_entry( struct relay_descr *descr, unsigned int idx,
     struct relay_private_data *data = descr->private;
     struct relay_entry_point *entry_point = data->entry_points + ordinal;
     unsigned int i, pos;
-#ifndef __SOFTFP__
     unsigned int float_pos = 0, double_pos = 0;
     const union fpregs { float s[16]; double d[8]; } *fpstack = (const union fpregs *)stack - 1;
-#endif
 
     TRACE( "\1Call %s(", func_name( data, ordinal ));
 
@@ -481,25 +474,21 @@ void * WINAPI relay_trace_entry( struct relay_descr *descr, unsigned int idx,
             trace_string_w( stack[pos++] );
             break;
         case 'f': /* float */
-#ifndef __SOFTFP__
             if (!(float_pos % 2)) float_pos = max( float_pos, double_pos * 2 );
             if (float_pos < 16)
             {
                 TRACE( "%g", fpstack->s[float_pos++] );
                 break;
             }
-#endif
             TRACE( "%g", *(const float *)&stack[pos++] );
             break;
         case 'd': /* double */
-#ifndef __SOFTFP__
             double_pos = max( (float_pos + 1) / 2, double_pos );
             if (double_pos < 8)
             {
                 TRACE( "%g", fpstack->d[double_pos++] );
                 break;
             }
-#endif
             pos = (pos + 1) & ~1;
             TRACE( "%g", *(const double *)&stack[pos] );
             pos += 2;
@@ -512,13 +501,11 @@ void * WINAPI relay_trace_entry( struct relay_descr *descr, unsigned int idx,
         if (!is_ret_val( arg_types[i+1] )) TRACE( "," );
     }
 
-#ifndef __SOFTFP__
     if (float_pos || double_pos)
     {
         pos |= 0x80000000;
         stack = (const DWORD *)fpstack;  /* retaddr is below the fp regs */
     }
-#endif
     *nb_args = pos;
     TRACE( ") ret=%08lx\n", stack[-1] );
     return entry_point->orig_func;
@@ -550,7 +537,7 @@ __ASM_GLOBAL_FUNC( relay_call,
                    "add r3, sp, #12\n\t"
                    "mov r7, r0\n\t"
                    "mov r8, r1\n\t"
-                   "bl " __ASM_NAME("relay_trace_entry") "\n\t"
+                   "bl relay_trace_entry\n\t"
                    "mov ip, r0\n\t"  /* entry point */
                    "mov r5, sp\n\t"
                    "ldr r1, [sp, #12]\n\t"  /* number of args */
@@ -566,14 +553,10 @@ __ASM_GLOBAL_FUNC( relay_call,
                    "str r0, [sp, r3]\n\t"
                    "bgt 1b\n"
                    "2:\t"
-#ifndef __SOFTFP__
                    "tst r1, #0x80000000\n\t"
                    "ldm r6, {r0-r3}\n\t"
                    "it ne\n\t"
                    "vldmdbne r6!, {s0-s15}\n\t"
-#else
-                   "ldm r6, {r0-r3}\n\t"
-#endif
                    "blx ip\n\t"
                    "mov sp, r5\n\t"
                    "ldr r2, [r6, #-4]\n\t"  /* retaddr */
@@ -583,13 +566,9 @@ __ASM_GLOBAL_FUNC( relay_call,
                    "mov r1, r8\n\t"
                    "str r4, [sp]\n\t"
                    "str r5, [sp, #4]\n\t"
-#ifndef __SOFTFP__
                    "vstr d0, [sp, #8]\n\t"  /* preserve floating point retval */
-                   "bl " __ASM_NAME("relay_trace_exit") "\n\t"
+                   "bl relay_trace_exit\n\t"
                    "vldr d0, [sp, #8]\n\t"
-#else
-                   "bl " __ASM_NAME("relay_trace_exit") "\n\t"
-#endif
                    "mov r0, r4\n\t"
                    "mov r1, r5\n\t"
                    "add sp, #16\n\t"
