@@ -263,15 +263,6 @@ void set_process_instrumentation_callback( void *callback )
 }
 
 
-/***********************************************************************
- *           unwind_builtin_dll
- */
-NTSTATUS unwind_builtin_dll( void *args )
-{
-    return STATUS_UNSUCCESSFUL;
-}
-
-
 /***********************************************************************
  *           get_trap_code
  *
