@@ -861,15 +861,6 @@ BOOL WINAPI AnimateWindow( HWND hwnd, DWORD time, DWORD flags )
 }
 
 
-/***********************************************************************
- *           BeginDeferWindowPos (USER32.@)
- */
-HDWP WINAPI BeginDeferWindowPos( INT count )
-{
-    return NtUserBeginDeferWindowPos( count );
-}
-
-
 /***********************************************************************
  *           DeferWindowPos (USER32.@)
  */
@@ -1502,12 +1493,7 @@ BOOL WINAPI UpdateLayeredWindow( HWND hwnd, HDC hdcDst, POINT *pptDst, SIZE *psi
  */
 BOOL WINAPI GetProcessDefaultLayout( DWORD *layout )
 {
-    if (!layout)
-    {
-        SetLastError( ERROR_NOACCESS );
-        return FALSE;
-    }
-    *layout = NtUserGetProcessDefaultLayout();
+    if (!NtUserGetProcessDefaultLayout( layout )) return FALSE;
     if (*layout == ~0u)
     {
         WCHAR *str, buffer[MAX_PATH];
