@@ -39,6 +39,7 @@ struct ime_update
     struct list entry;
     WORD vkey;
     WORD scan;
+    BOOL key_consumed;
     DWORD cursor_pos;
     WCHAR *comp_str;
     WCHAR *result_str;
@@ -440,16 +441,35 @@ static void post_ime_update( HWND hwnd, UINT cursor_pos, WCHAR *comp_str, WCHAR
     static UINT ime_update_count;
 
     struct imm_thread_data *data = get_imm_thread_data();
-    UINT id = -1, comp_len, result_len;
+    UINT id = -1, comp_len, result_len, prev_result_len;
+    WCHAR *prev_result_str, *tmp;
     struct ime_update *update;
 
-    TRACE( "hwnd %p, cursor_pos %u, comp_str %s, result_str %s\n", hwnd, cursor_pos,
-           debugstr_w(comp_str), debugstr_w(result_str) );
+    TRACE( "hwnd %p, cursor_pos %u - %u, comp_str %s, result_str %s\n", hwnd, LOWORD(cursor_pos),
+           HIWORD(cursor_pos), debugstr_w(comp_str), debugstr_w(result_str) );
 
     comp_len = comp_str ? wcslen( comp_str ) + 1 : 0;
     result_len = result_str ? wcslen( result_str ) + 1 : 0;
 
-    if (!(update = malloc( offsetof(struct ime_update, buffer[comp_len + result_len]) ))) return;
+    /* prepend or keep the previous result string, if there was any */
+    if (!data->ime_process_vkey || !data->update) prev_result_str = NULL;
+    else prev_result_str = data->update->result_str;
+    prev_result_len = prev_result_str ? wcslen( prev_result_str ) + 1 : 0;
+
+    if (!prev_result_len && !result_len) tmp = NULL;
+    else if (!(tmp = malloc( (prev_result_len + result_len) * sizeof(WCHAR) ))) return;
+
+    if (prev_result_len && result_len) prev_result_len -= 1; /* concat both strings */
+    if (prev_result_len) memcpy( tmp, prev_result_str, prev_result_len * sizeof(WCHAR) );
+    if (result_len) memcpy( tmp + prev_result_len, result_str, result_len * sizeof(WCHAR) );
+    result_len += prev_result_len;
+    result_str = tmp;
+
+    if (!(update = malloc( offsetof(struct ime_update, buffer[comp_len + result_len]) )))
+    {
+        free( tmp );
+        return;
+    }
     update->cursor_pos = cursor_pos;
     update->comp_str = comp_str ? memcpy( update->buffer, comp_str, comp_len * sizeof(WCHAR) ) : NULL;
     update->result_str = result_str ? memcpy( update->buffer + comp_len, result_str, result_len * sizeof(WCHAR) ) : NULL;
@@ -459,10 +479,11 @@ static void post_ime_update( HWND hwnd, UINT cursor_pos, WCHAR *comp_str, WCHAR
         pthread_mutex_lock( &imm_mutex );
         id = update->scan = ++ime_update_count;
         update->vkey = VK_PROCESSKEY;
+        update->key_consumed = TRUE;
         list_add_tail( &ime_updates, &update->entry );
         pthread_mutex_unlock( &imm_mutex );
 
-        NtUserPostMessage( hwnd, WM_IME_NOTIFY, IMN_WINE_SET_COMP_STRING, id );
+        NtUserPostMessage( hwnd, WM_WINE_IME_NOTIFY, IMN_WINE_SET_COMP_STRING, id );
     }
     else
     {
@@ -470,6 +491,50 @@ static void post_ime_update( HWND hwnd, UINT cursor_pos, WCHAR *comp_str, WCHAR
         free( data->update );
         data->update = update;
     }
+
+    free( tmp );
+}
+
+static UINT get_comp_clause_count( UINT comp_len, UINT cursor_begin, UINT cursor_end )
+{
+    if (cursor_begin == cursor_end || (cursor_begin == 0 && cursor_end == comp_len))
+        return 2;
+    else if (cursor_begin == 0 || cursor_end == comp_len)
+        return 3;
+    else
+        return 4;
+}
+
+static void set_comp_clause( DWORD *comp_clause, UINT comp_clause_count, UINT comp_len,
+                             UINT cursor_begin, UINT cursor_end )
+{
+    comp_clause[0] = 0;
+    switch (comp_clause_count)
+    {
+    case 2:
+        comp_clause[1] = comp_len;
+        break;
+    case 3:
+        comp_clause[1] = cursor_begin == 0 ? cursor_end : cursor_begin;
+        comp_clause[2] = comp_len;
+        break;
+    case 4:
+        comp_clause[1] = cursor_begin;
+        comp_clause[2] = cursor_end;
+        comp_clause[3] = comp_len;
+        break;
+    }
+}
+
+static void set_comp_attr( BYTE *comp_attr, UINT comp_attr_len, UINT cursor_begin, UINT cursor_end )
+{
+    if (cursor_begin == cursor_end)
+        memset( comp_attr, ATTR_INPUT, comp_attr_len );
+    else
+    {
+        memset( comp_attr, ATTR_CONVERTED, comp_attr_len );
+        memset( comp_attr + cursor_begin, ATTR_TARGET_CONVERTED, cursor_end - cursor_begin );
+    }
 }
 
 static struct ime_update *find_ime_update( WORD vkey, WORD scan )
@@ -482,9 +547,11 @@ static struct ime_update *find_ime_update( WORD vkey, WORD scan )
     return NULL;
 }
 
-UINT ime_to_tascii_ex( UINT vkey, UINT lparam, const BYTE *state, COMPOSITIONSTRING *compstr, HIMC himc )
+static UINT ime_to_tascii_ex( UINT vkey, UINT lparam, const BYTE *state, COMPOSITIONSTRING *compstr,
+                              BOOL *key_consumed, HIMC himc )
 {
-    UINT needed = sizeof(COMPOSITIONSTRING), comp_len, result_len;
+    UINT needed = sizeof(COMPOSITIONSTRING), comp_len, result_len, comp_clause_count = 0;
+    UINT cursor_begin = 0, cursor_end = 0;
     struct ime_update *update;
     void *dst;
 
@@ -498,13 +565,24 @@ UINT ime_to_tascii_ex( UINT vkey, UINT lparam, const BYTE *state, COMPOSITIONSTR
         return STATUS_NOT_FOUND;
     }
 
+    *key_consumed = update->key_consumed;
+
     if (!update->comp_str) comp_len = 0;
     else
     {
         comp_len = wcslen( update->comp_str );
+        cursor_begin = LOWORD(update->cursor_pos);
+        cursor_end   = HIWORD(update->cursor_pos);
+
+        if (cursor_begin > comp_len) cursor_begin = comp_len;
+        if (cursor_end > comp_len) cursor_end = comp_len;
+        if (cursor_end < cursor_begin) cursor_end = cursor_begin;
+
+        comp_clause_count = get_comp_clause_count( comp_len, cursor_begin, cursor_end );
+
         needed += comp_len * sizeof(WCHAR); /* GCS_COMPSTR */
         needed += comp_len; /* GCS_COMPATTR */
-        needed += 2 * sizeof(DWORD); /* GCS_COMPCLAUSE */
+        needed += comp_clause_count * sizeof(DWORD); /* GCS_COMPCLAUSE */
     }
 
     if (!update->result_str) result_len = 0;
@@ -530,7 +608,7 @@ UINT ime_to_tascii_ex( UINT vkey, UINT lparam, const BYTE *state, COMPOSITIONSTR
 
     if (update->comp_str)
     {
-        compstr->dwCursorPos = update->cursor_pos;
+        compstr->dwCursorPos = cursor_begin;
 
         compstr->dwCompStrLen = comp_len;
         compstr->dwCompStrOffset = compstr->dwSize;
@@ -538,17 +616,16 @@ UINT ime_to_tascii_ex( UINT vkey, UINT lparam, const BYTE *state, COMPOSITIONSTR
         memcpy( dst, update->comp_str, compstr->dwCompStrLen * sizeof(WCHAR) );
         compstr->dwSize += compstr->dwCompStrLen * sizeof(WCHAR);
 
-        compstr->dwCompClauseLen = 2 * sizeof(DWORD);
+        compstr->dwCompClauseLen = comp_clause_count * sizeof(DWORD);
         compstr->dwCompClauseOffset = compstr->dwSize;
         dst = (BYTE *)compstr + compstr->dwCompClauseOffset;
-        *((DWORD *)dst + 0) = 0;
-        *((DWORD *)dst + 1) = compstr->dwCompStrLen;
+        set_comp_clause( dst, comp_clause_count, comp_len, cursor_begin, cursor_end );
         compstr->dwSize += compstr->dwCompClauseLen;
 
         compstr->dwCompAttrLen = compstr->dwCompStrLen;
         compstr->dwCompAttrOffset = compstr->dwSize;
         dst = (BYTE *)compstr + compstr->dwCompAttrOffset;
-        memset( dst, ATTR_INPUT, compstr->dwCompAttrLen );
+        set_comp_attr( dst, compstr->dwCompAttrLen, cursor_begin, cursor_end );
         compstr->dwSize += compstr->dwCompAttrLen;
     }
 
@@ -590,6 +667,7 @@ LRESULT ime_driver_call( HWND hwnd, enum wine_ime_call call, WPARAM wparam, LPAR
 
         if (data->update)
         {
+            data->update->key_consumed = res;
             pthread_mutex_lock( &imm_mutex );
             list_add_tail( &ime_updates, &data->update->entry );
             pthread_mutex_unlock( &imm_mutex );
@@ -601,7 +679,7 @@ LRESULT ime_driver_call( HWND hwnd, enum wine_ime_call call, WPARAM wparam, LPAR
         return res;
     }
     case WINE_IME_TO_ASCII_EX:
-        return ime_to_tascii_ex( wparam, lparam, params->state, params->compstr, params->himc );
+        return ime_to_tascii_ex( wparam, lparam, params->state, params->compstr, params->key_consumed, params->himc );
     case WINE_IME_POST_UPDATE:
         post_ime_update( hwnd, wparam, (WCHAR *)lparam, (WCHAR *)params );
         return 0;
