@@ -78,17 +78,13 @@ struct image_descriptor
 
 #include "poppack.h"
 
-static HRESULT load_LSD_metadata(IStream *stream, const GUID *vendor, DWORD options,
-                                 MetadataItem **items, DWORD *count)
+static HRESULT load_LSD_metadata(MetadataHandler *handler, IStream *stream, const GUID *vendor, DWORD options)
 {
     struct logical_screen_descriptor lsd_data;
     HRESULT hr;
     ULONG bytesread, i;
     MetadataItem *result;
 
-    *items = NULL;
-    *count = 0;
-
     hr = IStream_Read(stream, &lsd_data, sizeof(lsd_data), &bytesread);
     if (FAILED(hr) || bytesread != sizeof(lsd_data)) return S_OK;
 
@@ -149,8 +145,9 @@ static HRESULT load_LSD_metadata(IStream *stream, const GUID *vendor, DWORD opti
     result[8].value.vt = VT_UI1;
     result[8].value.bVal = lsd_data.pixel_aspect_ratio;
 
-    *items = result;
-    *count = 9;
+    MetadataHandler_FreeItems(handler);
+    handler->items = result;
+    handler->item_count = 9;
 
     return S_OK;
 }
@@ -166,17 +163,13 @@ HRESULT LSDReader_CreateInstance(REFIID iid, void **ppv)
     return MetadataReader_Create(&LSDReader_Vtbl, iid, ppv);
 }
 
-static HRESULT load_IMD_metadata(IStream *stream, const GUID *vendor, DWORD options,
-                                 MetadataItem **items, DWORD *count)
+static HRESULT load_IMD_metadata(MetadataHandler *handler, IStream *stream, const GUID *vendor, DWORD options)
 {
     struct image_descriptor imd_data;
     HRESULT hr;
     ULONG bytesread, i;
     MetadataItem *result;
 
-    *items = NULL;
-    *count = 0;
-
     hr = IStream_Read(stream, &imd_data, sizeof(imd_data), &bytesread);
     if (FAILED(hr) || bytesread != sizeof(imd_data)) return S_OK;
 
@@ -230,8 +223,9 @@ static HRESULT load_IMD_metadata(IStream *stream, const GUID *vendor, DWORD opti
     result[7].value.vt = VT_UI1;
     result[7].value.bVal = imd_data.packed & 7;
 
-    *items = result;
-    *count = 8;
+    MetadataHandler_FreeItems(handler);
+    handler->items = result;
+    handler->item_count = 8;
 
     return S_OK;
 }
@@ -247,8 +241,7 @@ HRESULT IMDReader_CreateInstance(REFIID iid, void **ppv)
     return MetadataReader_Create(&IMDReader_Vtbl, iid, ppv);
 }
 
-static HRESULT load_GCE_metadata(IStream *stream, const GUID *vendor, DWORD options,
-                                 MetadataItem **items, DWORD *count)
+static HRESULT load_GCE_metadata(MetadataHandler *handler, IStream *stream, const GUID *vendor, DWORD options)
 {
 #include "pshpack1.h"
     struct graphic_control_extension
@@ -267,9 +260,6 @@ static HRESULT load_GCE_metadata(IStream *stream, const GUID *vendor, DWORD opti
     ULONG bytesread, i;
     MetadataItem *result;
 
-    *items = NULL;
-    *count = 0;
-
     hr = IStream_Read(stream, &gce_data, sizeof(gce_data), &bytesread);
     if (FAILED(hr) || bytesread != sizeof(gce_data)) return S_OK;
 
@@ -308,8 +298,9 @@ static HRESULT load_GCE_metadata(IStream *stream, const GUID *vendor, DWORD opti
     result[4].value.vt = VT_UI1;
     result[4].value.bVal = gce_data.transparent_color_index;
 
-    *items = result;
-    *count = 5;
+    MetadataHandler_FreeItems(handler);
+    handler->items = result;
+    handler->item_count = 5;
 
     return S_OK;
 }
@@ -325,8 +316,7 @@ HRESULT GCEReader_CreateInstance(REFIID iid, void **ppv)
     return MetadataReader_Create(&GCEReader_Vtbl, iid, ppv);
 }
 
-static HRESULT load_APE_metadata(IStream *stream, const GUID *vendor, DWORD options,
-                                 MetadataItem **items, DWORD *count)
+static HRESULT load_APE_metadata(MetadataHandler *handler, IStream *stream, const GUID *vendor, DWORD options)
 {
 #include "pshpack1.h"
     struct application_extension
@@ -343,9 +333,6 @@ static HRESULT load_APE_metadata(IStream *stream, const GUID *vendor, DWORD opti
     BYTE subblock_size;
     BYTE *data;
 
-    *items = NULL;
-    *count = 0;
-
     hr = IStream_Read(stream, &ape_data, sizeof(ape_data), &bytesread);
     if (FAILED(hr) || bytesread != sizeof(ape_data)) return S_OK;
     if (ape_data.extension_introducer != 0x21 ||
@@ -415,8 +402,9 @@ static HRESULT load_APE_metadata(IStream *stream, const GUID *vendor, DWORD opti
     result[1].value.caub.cElems = data_size;
     result[1].value.caub.pElems = data;
 
-    *items = result;
-    *count = 2;
+    MetadataHandler_FreeItems(handler);
+    handler->items = result;
+    handler->item_count = 2;
 
     return S_OK;
 }
@@ -432,8 +420,27 @@ HRESULT APEReader_CreateInstance(REFIID iid, void **ppv)
     return MetadataReader_Create(&APEReader_Vtbl, iid, ppv);
 }
 
-static HRESULT load_GifComment_metadata(IStream *stream, const GUID *vendor, DWORD options,
-                                        MetadataItem **items, DWORD *count)
+static HRESULT create_gifcomment_item(char *data, MetadataItem **item)
+{
+    HRESULT hr;
+
+    if (!(*item = calloc(1, sizeof(**item))))
+        return E_OUTOFMEMORY;
+
+    hr = init_propvar_from_string(L"TextEntry", &(*item)->id);
+    if (FAILED(hr))
+    {
+        free(*item);
+        return hr;
+    }
+
+    (*item)->value.vt = VT_LPSTR;
+    (*item)->value.pszVal = data;
+
+    return S_OK;
+}
+
+static HRESULT load_GifComment_metadata(MetadataHandler *handler, IStream *stream, const GUID *vendor, DWORD options)
 {
 #include "pshpack1.h"
     struct gif_extension
@@ -448,9 +455,6 @@ static HRESULT load_GifComment_metadata(IStream *stream, const GUID *vendor, DWO
     BYTE subblock_size;
     char *data;
 
-    *items = NULL;
-    *count = 0;
-
     hr = IStream_Read(stream, &ext_data, sizeof(ext_data), &bytesread);
     if (FAILED(hr) || bytesread != sizeof(ext_data)) return S_OK;
     if (ext_data.extension_introducer != 0x21 ||
@@ -493,24 +497,37 @@ static HRESULT load_GifComment_metadata(IStream *stream, const GUID *vendor, DWO
 
     data[data_size] = 0;
 
-    result = calloc(1, sizeof(MetadataItem));
-    if (!result)
+    if (FAILED(hr = create_gifcomment_item(data, &result)))
     {
         CoTaskMemFree(data);
-        return E_OUTOFMEMORY;
+        return hr;
     }
 
-    PropVariantInit(&result->schema);
-    PropVariantInit(&result->id);
-    PropVariantInit(&result->value);
+    MetadataHandler_FreeItems(handler);
+    handler->items = result;
+    handler->item_count = 1;
 
-    result->id.vt = VT_LPWSTR;
-    SHStrDupW(L"TextEntry", &result->id.pwszVal);
-    result->value.vt = VT_LPSTR;
-    result->value.pszVal = data;
+    return S_OK;
+}
 
-    *items = result;
-    *count = 1;
+static HRESULT CreateGifCommentHandler(MetadataHandler *handler)
+{
+    MetadataItem *item;
+    char *data;
+    HRESULT hr;
+
+    if (!(data = CoTaskMemAlloc(1)))
+        return E_OUTOFMEMORY;
+    *data = 0;
+
+    if (FAILED(hr = create_gifcomment_item(data, &item)))
+    {
+        CoTaskMemFree(data);
+        return hr;
+    }
+
+    handler->items = item;
+    handler->item_count = 1;
 
     return S_OK;
 }
@@ -518,7 +535,8 @@ static HRESULT load_GifComment_metadata(IStream *stream, const GUID *vendor, DWO
 static const MetadataHandlerVtbl GifCommentReader_Vtbl = {
     0,
     &CLSID_WICGifCommentMetadataReader,
-    load_GifComment_metadata
+    load_GifComment_metadata,
+    CreateGifCommentHandler,
 };
 
 HRESULT GifCommentReader_CreateInstance(REFIID iid, void **ppv)
