@@ -39,6 +39,8 @@
 #include <EGL/egl.h>
 #endif
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "android.h"
 #include "winternl.h"
 
@@ -70,7 +72,7 @@ struct egl_pixel_format
     EGLConfig config;
 };
 
-struct wgl_context
+struct android_context
 {
     struct list entry;
     EGLConfig  config;
@@ -108,11 +110,6 @@ static void (*pglFlush)(void);
 
 pthread_mutex_t drawable_mutex;
 
-static inline BOOL is_onscreen_pixel_format( int format )
-{
-    return format > 0 && format <= nb_onscreen_formats;
-}
-
 static struct gl_drawable *create_gl_drawable( HWND hwnd, HDC hdc, int format )
 {
     static const int attribs[] = { EGL_WIDTH, 1, EGL_HEIGHT, 1, EGL_NONE };
@@ -166,7 +163,7 @@ void destroy_gl_drawable( HWND hwnd )
     pthread_mutex_unlock( &drawable_mutex );
 }
 
-static BOOL refresh_context( struct wgl_context *ctx )
+static BOOL refresh_context( struct android_context *ctx )
 {
     BOOL ret = InterlockedExchange( &ctx->refresh, FALSE );
 
@@ -182,19 +179,19 @@ static BOOL refresh_context( struct wgl_context *ctx )
 void update_gl_drawable( HWND hwnd )
 {
     struct gl_drawable *gl;
-    struct wgl_context *ctx;
+    struct android_context *ctx;
 
     if ((gl = get_gl_drawable( hwnd, 0 )))
     {
         if (!gl->surface &&
             (gl->surface = p_eglCreateWindowSurface( display, pixel_formats[gl->format - 1].config, gl->window, NULL )))
         {
-            LIST_FOR_EACH_ENTRY( ctx, &gl_contexts, struct wgl_context, entry )
+            LIST_FOR_EACH_ENTRY( ctx, &gl_contexts, struct android_context, entry )
             {
                 if (ctx->hwnd != hwnd) continue;
-                TRACE( "hwnd %p refreshing %p %scurrent\n", hwnd, ctx, NtCurrentTeb()->glContext == ctx ? "" : "not " );
+                TRACE( "hwnd %p refreshing %p %scurrent\n", hwnd, ctx, NtCurrentTeb()->glReserved2 == ctx ? "" : "not " );
                 ctx->surface = gl->surface;
-                if (NtCurrentTeb()->glContext == ctx)
+                if (NtCurrentTeb()->glReserved2 == ctx)
                     p_eglMakeCurrent( display, ctx->surface, ctx->surface, ctx->context );
                 else
                     InterlockedExchange( &ctx->refresh, TRUE );
@@ -205,76 +202,87 @@ void update_gl_drawable( HWND hwnd )
     }
 }
 
-static BOOL set_pixel_format( HDC hdc, int format, BOOL internal )
+static BOOL android_set_pixel_format( HWND hwnd, int old_format, int new_format, BOOL internal )
 {
     struct gl_drawable *gl;
-    HWND hwnd = NtUserWindowFromDC( hdc );
 
-    if (!hwnd || hwnd == NtUserGetDesktopWindow())
-    {
-        WARN( "not a proper window DC %p/%p\n", hdc, hwnd );
-        return FALSE;
-    }
-    if (!is_onscreen_pixel_format( format ))
-    {
-        WARN( "Invalid format %d\n", format );
-        return FALSE;
-    }
-    TRACE( "%p/%p format %d\n", hdc, hwnd, format );
-
-    if (!internal)
-    {
-        /* cannot change it if already set */
-        int prev = win32u_get_window_pixel_format( hwnd );
-
-        if (prev)
-            return prev == format;
-    }
+    TRACE( "hwnd %p, old_format %d, new_format %d, internal %u\n", hwnd, old_format, new_format, internal );
 
     if ((gl = get_gl_drawable( hwnd, 0 )))
     {
         if (internal)
         {
             EGLint pf;
-            p_eglGetConfigAttrib( display, pixel_formats[format - 1].config, EGL_NATIVE_VISUAL_ID, &pf );
+            p_eglGetConfigAttrib( display, pixel_formats[new_format - 1].config, EGL_NATIVE_VISUAL_ID, &pf );
             gl->window->perform( gl->window, NATIVE_WINDOW_SET_BUFFERS_FORMAT, pf );
-            gl->format = format;
+            gl->format = new_format;
         }
     }
-    else gl = create_gl_drawable( hwnd, 0, format );
-
+    else gl = create_gl_drawable( hwnd, 0, new_format );
     release_gl_drawable( gl );
 
-    if (win32u_set_window_pixel_format( hwnd, format, internal )) return TRUE;
-    destroy_gl_drawable( hwnd );
-    return FALSE;
+    return TRUE;
 }
 
-static struct wgl_context *create_context( HDC hdc, struct wgl_context *share, const int *attribs )
+static BOOL android_context_create( HDC hdc, int format, void *share, const int *attribs, void **private )
 {
-    struct gl_drawable *gl;
-    struct wgl_context *ctx;
+    struct android_context *ctx, *shared_ctx = share;
+    int count = 0, egl_attribs[3];
+    BOOL opengl_es = FALSE;
 
-    if (!(gl = get_gl_drawable( NtUserWindowFromDC( hdc ), hdc ))) return NULL;
+    if (!attribs) opengl_es = TRUE;
+    while (attribs && *attribs && count < 2)
+    {
+        switch (*attribs)
+        {
+        case WGL_CONTEXT_PROFILE_MASK_ARB:
+            if (attribs[1] == WGL_CONTEXT_ES2_PROFILE_BIT_EXT)
+                opengl_es = TRUE;
+            break;
+        case WGL_CONTEXT_MAJOR_VERSION_ARB:
+            egl_attribs[count++] = EGL_CONTEXT_CLIENT_VERSION;
+            egl_attribs[count++] = attribs[1];
+            break;
+        default:
+            FIXME("Unhandled attributes: %#x %#x\n", attribs[0], attribs[1]);
+        }
+        attribs += 2;
+    }
+    if (!opengl_es)
+    {
+        WARN("Requested creation of an OpenGL (non ES) context, that's not supported.\n");
+        return FALSE;
+    }
+    if (!count)  /* FIXME: force version if not specified */
+    {
+        egl_attribs[count++] = EGL_CONTEXT_CLIENT_VERSION;
+        egl_attribs[count++] = egl_client_version;
+    }
+    egl_attribs[count] = EGL_NONE;
+    attribs = egl_attribs;
 
     ctx = malloc( sizeof(*ctx) );
 
-    ctx->config  = pixel_formats[gl->format - 1].config;
+    ctx->config  = pixel_formats[format - 1].config;
     ctx->surface = 0;
     ctx->refresh = FALSE;
-    ctx->context = p_eglCreateContext( display, ctx->config,
-                                       share ? share->context : EGL_NO_CONTEXT, attribs );
-    TRACE( "%p fmt %d ctx %p\n", hdc, gl->format, ctx->context );
+    ctx->context = p_eglCreateContext( display, ctx->config, shared_ctx ? shared_ctx->context : EGL_NO_CONTEXT, attribs );
+    TRACE( "%p fmt %d ctx %p\n", hdc, format, ctx->context );
     list_add_head( &gl_contexts, &ctx->entry );
-    release_gl_drawable( gl );
-    return ctx;
+
+    *private = ctx;
+    return TRUE;
 }
 
-static void describe_pixel_format( struct egl_pixel_format *fmt, PIXELFORMATDESCRIPTOR *pfd )
+static BOOL android_describe_pixel_format( int format, struct wgl_pixel_format *desc )
 {
+    struct egl_pixel_format *fmt = pixel_formats + format - 1;
+    PIXELFORMATDESCRIPTOR *pfd = &desc->pfd;
     EGLint val;
     EGLConfig config = fmt->config;
 
+    if (format <= 0 || format > nb_pixel_formats) return FALSE;
+
     memset( pfd, 0, sizeof(*pfd) );
     pfd->nSize = sizeof(*pfd);
     pfd->nVersion = 1;
@@ -301,72 +309,12 @@ static void describe_pixel_format( struct egl_pixel_format *fmt, PIXELFORMATDESC
     pfd->cBlueShift = pfd->cAlphaShift + pfd->cAlphaBits;
     pfd->cGreenShift = pfd->cBlueShift + pfd->cBlueBits;
     pfd->cRedShift = pfd->cGreenShift + pfd->cGreenBits;
+    return TRUE;
 }
 
-/***********************************************************************
- *		android_wglGetExtensionsStringARB
- */
-static const char *android_wglGetExtensionsStringARB( HDC hdc )
-{
-    TRACE( "() returning \"%s\"\n", wgl_extensions );
-    return wgl_extensions;
-}
-
-/***********************************************************************
- *		android_wglGetExtensionsStringEXT
- */
-static const char *android_wglGetExtensionsStringEXT(void)
-{
-    TRACE( "() returning \"%s\"\n", wgl_extensions );
-    return wgl_extensions;
-}
-
-/***********************************************************************
- *		android_wglCreateContextAttribsARB
- */
-static struct wgl_context *android_wglCreateContextAttribsARB( HDC hdc, struct wgl_context *share,
-                                                               const int *attribs )
-{
-    int count = 0, egl_attribs[3];
-    BOOL opengl_es = FALSE;
-
-    while (attribs && *attribs && count < 2)
-    {
-        switch (*attribs)
-        {
-        case WGL_CONTEXT_PROFILE_MASK_ARB:
-            if (attribs[1] == WGL_CONTEXT_ES2_PROFILE_BIT_EXT)
-                opengl_es = TRUE;
-            break;
-        case WGL_CONTEXT_MAJOR_VERSION_ARB:
-            egl_attribs[count++] = EGL_CONTEXT_CLIENT_VERSION;
-            egl_attribs[count++] = attribs[1];
-            break;
-        default:
-            FIXME("Unhandled attributes: %#x %#x\n", attribs[0], attribs[1]);
-        }
-        attribs += 2;
-    }
-    if (!opengl_es)
-    {
-        WARN("Requested creation of an OpenGL (non ES) context, that's not supported.\n");
-        return NULL;
-    }
-    if (!count)  /* FIXME: force version if not specified */
-    {
-        egl_attribs[count++] = EGL_CONTEXT_CLIENT_VERSION;
-        egl_attribs[count++] = egl_client_version;
-    }
-    egl_attribs[count] = EGL_NONE;
-
-    return create_context( hdc, share, egl_attribs );
-}
-
-/***********************************************************************
- *		android_wglMakeContextCurrentARB
- */
-static BOOL android_wglMakeContextCurrentARB( HDC draw_hdc, HDC read_hdc, struct wgl_context *ctx )
+static BOOL android_context_make_current( HDC draw_hdc, HDC read_hdc, void *private )
 {
+    struct android_context *ctx = private;
     BOOL ret = FALSE;
     struct gl_drawable *draw_gl, *read_gl = NULL;
     EGLSurface draw_surface, read_surface;
@@ -374,10 +322,10 @@ static BOOL android_wglMakeContextCurrentARB( HDC draw_hdc, HDC read_hdc, struct
 
     TRACE( "%p %p %p\n", draw_hdc, read_hdc, ctx );
 
-    if (!ctx)
+    if (!private)
     {
         p_eglMakeCurrent( display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT );
-        NtCurrentTeb()->glContext = NULL;
+        NtCurrentTeb()->glReserved2 = NULL;
         return TRUE;
     }
 
@@ -395,7 +343,7 @@ static BOOL android_wglMakeContextCurrentARB( HDC draw_hdc, HDC read_hdc, struct
             ctx->surface = draw_gl->surface;
             ctx->hwnd    = draw_hwnd;
             ctx->refresh = FALSE;
-            NtCurrentTeb()->glContext = ctx;
+            NtCurrentTeb()->glReserved2 = ctx;
             goto done;
         }
     }
@@ -440,38 +388,21 @@ static int android_wglGetSwapIntervalEXT(void)
     return swap_interval;
 }
 
-/***********************************************************************
- *		android_wglSetPixelFormatWINE
- */
-static BOOL android_wglSetPixelFormatWINE( HDC hdc, int format )
-{
-    return set_pixel_format( hdc, format, TRUE );
-}
-
 /***********************************************************************
  *		android_wglCopyContext
  */
-static BOOL android_wglCopyContext( struct wgl_context *src, struct wgl_context *dst, UINT mask )
+static BOOL android_context_copy( void *src, void *dst, UINT mask )
 {
     FIXME( "%p -> %p mask %#x unsupported\n", src, dst, mask );
     return FALSE;
 }
 
-/***********************************************************************
- *		android_wglCreateContext
- */
-static struct wgl_context *android_wglCreateContext( HDC hdc )
-{
-    int egl_attribs[3] = { EGL_CONTEXT_CLIENT_VERSION, egl_client_version, EGL_NONE };
-
-    return create_context( hdc, NULL, egl_attribs );
-}
-
 /***********************************************************************
  *		android_wglDeleteContext
  */
-static BOOL android_wglDeleteContext( struct wgl_context *ctx )
+static BOOL android_context_destroy( void *private )
 {
+    struct android_context *ctx = private;
     pthread_mutex_lock( &drawable_mutex );
     list_remove( &ctx->entry );
     pthread_mutex_unlock( &drawable_mutex );
@@ -480,30 +411,6 @@ static BOOL android_wglDeleteContext( struct wgl_context *ctx )
     return TRUE;
 }
 
-/***********************************************************************
- *		android_wglGetPixelFormat
- */
-static int android_wglGetPixelFormat( HDC hdc )
-{
-    struct gl_drawable *gl;
-    int ret = 0;
-    HWND hwnd;
-
-    if ((hwnd = NtUserWindowFromDC( hdc )))
-        return win32u_get_window_pixel_format( hwnd );
-
-    /* This code is currently dead, but will be necessary if WGL_ARB_pbuffer
-     * support is introduced. */
-    if ((gl = get_gl_drawable( NULL, hdc )))
-    {
-        ret = gl->format;
-        /* offscreen formats can't be used with traditional WGL calls */
-        if (!is_onscreen_pixel_format( ret )) ret = 1;
-        release_gl_drawable( gl );
-    }
-    return ret;
-}
-
 /***********************************************************************
  *		android_wglGetProcAddress
  */
@@ -516,58 +423,7 @@ static PROC android_wglGetProcAddress( LPCSTR name )
     return ret;
 }
 
-/***********************************************************************
- *		android_wglMakeCurrent
- */
-static BOOL android_wglMakeCurrent( HDC hdc, struct wgl_context *ctx )
-{
-    BOOL ret = FALSE;
-    struct gl_drawable *gl;
-    HWND hwnd;
-
-    TRACE( "%p %p\n", hdc, ctx );
-
-    if (!ctx)
-    {
-        p_eglMakeCurrent( display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT );
-        NtCurrentTeb()->glContext = NULL;
-        return TRUE;
-    }
-
-    hwnd = NtUserWindowFromDC( hdc );
-    if ((gl = get_gl_drawable( hwnd, hdc )))
-    {
-        EGLSurface surface = gl->surface ? gl->surface : gl->pbuffer;
-        TRACE( "%p hwnd %p context %p surface %p\n", hdc, gl->hwnd, ctx->context, surface );
-        ret = p_eglMakeCurrent( display, surface, surface, ctx->context );
-        if (ret)
-        {
-            ctx->surface = gl->surface;
-            ctx->hwnd    = hwnd;
-            ctx->refresh = FALSE;
-            NtCurrentTeb()->glContext = ctx;
-            goto done;
-        }
-    }
-    RtlSetLastWin32Error( ERROR_INVALID_HANDLE );
-
-done:
-    release_gl_drawable( gl );
-    return ret;
-}
-
-/***********************************************************************
- *		android_wglSetPixelFormat
- */
-static BOOL android_wglSetPixelFormat( HDC hdc, int format, const PIXELFORMATDESCRIPTOR *pfd )
-{
-    return set_pixel_format( hdc, format, FALSE );
-}
-
-/***********************************************************************
- *		android_wglShareLists
- */
-static BOOL android_wglShareLists( struct wgl_context *org, struct wgl_context *dest )
+static BOOL android_context_share( void *org, void *dest )
 {
     FIXME( "%p %p\n", org, dest );
     return FALSE;
@@ -578,7 +434,7 @@ static BOOL android_wglShareLists( struct wgl_context *org, struct wgl_context *
  */
 static BOOL android_wglSwapBuffers( HDC hdc )
 {
-    struct wgl_context *ctx = NtCurrentTeb()->glContext;
+    struct android_context *ctx = NtCurrentTeb()->glReserved2;
 
     if (!ctx) return FALSE;
 
@@ -589,27 +445,9 @@ static BOOL android_wglSwapBuffers( HDC hdc )
     return TRUE;
 }
 
-/**********************************************************************
- *              android_get_pixel_formats
- */
-static void android_get_pixel_formats( struct wgl_pixel_format *formats,
-                                       UINT max_formats, UINT *num_formats,
-                                       UINT *num_onscreen_formats )
-{
-    UINT i;
-
-    if (formats)
-    {
-        for (i = 0; i < min( max_formats, nb_pixel_formats ); ++i)
-            describe_pixel_format( &pixel_formats[i], &formats[i].pfd );
-    }
-    *num_formats = nb_pixel_formats;
-    *num_onscreen_formats = nb_onscreen_formats;
-}
-
 static void wglFinish(void)
 {
-    struct wgl_context *ctx = NtCurrentTeb()->glContext;
+    struct android_context *ctx = NtCurrentTeb()->glReserved2;
 
     if (!ctx) return;
     TRACE( "hwnd %p context %p\n", ctx->hwnd, ctx->context );
@@ -619,7 +457,7 @@ static void wglFinish(void)
 
 static void wglFlush(void)
 {
-    struct wgl_context *ctx = NtCurrentTeb()->glContext;
+    struct android_context *ctx = NtCurrentTeb()->glReserved2;
 
     if (!ctx) return;
     TRACE( "hwnd %p context %p\n", ctx->hwnd, ctx->context );
@@ -634,35 +472,19 @@ static void register_extension( const char *ext )
     TRACE( "%s\n", ext );
 }
 
-static void init_extensions(void)
+static const char *android_init_wgl_extensions(void)
 {
-    void *ptr;
-
-    register_extension("WGL_ARB_create_context");
-    register_extension("WGL_ARB_create_context_profile");
-    egl_funcs.p_wglCreateContextAttribsARB = android_wglCreateContextAttribsARB;
-
-    register_extension("WGL_ARB_extensions_string");
-    egl_funcs.p_wglGetExtensionsStringARB = android_wglGetExtensionsStringARB;
-
-    register_extension("WGL_ARB_make_current_read");
-    egl_funcs.p_wglGetCurrentReadDCARB   = (void *)1;  /* never called */
-    egl_funcs.p_wglMakeContextCurrentARB = android_wglMakeContextCurrentARB;
-
-    register_extension("WGL_EXT_extensions_string");
-    egl_funcs.p_wglGetExtensionsStringEXT = android_wglGetExtensionsStringEXT;
-
     register_extension("WGL_EXT_swap_control");
     egl_funcs.p_wglSwapIntervalEXT = android_wglSwapIntervalEXT;
     egl_funcs.p_wglGetSwapIntervalEXT = android_wglGetSwapIntervalEXT;
 
     register_extension("WGL_EXT_framebuffer_sRGB");
+    return wgl_extensions;
+}
 
-    /* In WineD3D we need the ability to set the pixel format more than once (e.g. after a device reset).
-     * The default wglSetPixelFormat doesn't allow this, so add our own which allows it.
-     */
-    register_extension("WGL_WINE_pixel_format_passthrough");
-    egl_funcs.p_wglSetPixelFormatWINE = android_wglSetPixelFormatWINE;
+static void init_opengl_funcs(void)
+{
+    void *ptr;
 
     /* load standard functions and extensions exported from the OpenGL library */
 
@@ -955,52 +777,10 @@ static void init_extensions(void)
 #undef REDIRECT
 }
 
-/**********************************************************************
- *           ANDROID_wine_get_wgl_driver
- */
-struct opengl_funcs *ANDROID_wine_get_wgl_driver( UINT version )
+static UINT android_init_pixel_formats( UINT *onscreen_count )
 {
     EGLConfig *configs;
-    EGLint major, minor, count, i, pass;
-
-    if (version != WINE_OPENGL_DRIVER_VERSION)
-    {
-        ERR( "version mismatch, opengl32 wants %u but driver has %u\n", version, WINE_OPENGL_DRIVER_VERSION );
-        return NULL;
-    }
-    if (!(egl_handle = dlopen( SONAME_LIBEGL, RTLD_NOW|RTLD_GLOBAL )))
-    {
-        ERR( "failed to load %s: %s\n", SONAME_LIBEGL, dlerror() );
-        return NULL;
-    }
-    if (!(opengl_handle = dlopen( SONAME_LIBGLESV2, RTLD_NOW|RTLD_GLOBAL )))
-    {
-        ERR( "failed to load %s: %s\n", SONAME_LIBGLESV2, dlerror() );
-        return NULL;
-    }
-
-#define LOAD_FUNCPTR(func) do { \
-        if (!(p_##func = dlsym( egl_handle, #func ))) \
-        { ERR( "can't find symbol %s\n", #func); return FALSE; }    \
-    } while(0)
-    LOAD_FUNCPTR( eglCreateContext );
-    LOAD_FUNCPTR( eglCreateWindowSurface );
-    LOAD_FUNCPTR( eglCreatePbufferSurface );
-    LOAD_FUNCPTR( eglDestroyContext );
-    LOAD_FUNCPTR( eglDestroySurface );
-    LOAD_FUNCPTR( eglGetConfigAttrib );
-    LOAD_FUNCPTR( eglGetConfigs );
-    LOAD_FUNCPTR( eglGetDisplay );
-    LOAD_FUNCPTR( eglGetProcAddress );
-    LOAD_FUNCPTR( eglInitialize );
-    LOAD_FUNCPTR( eglMakeCurrent );
-    LOAD_FUNCPTR( eglSwapBuffers );
-    LOAD_FUNCPTR( eglSwapInterval );
-#undef LOAD_FUNCPTR
-
-    display = p_eglGetDisplay( EGL_DEFAULT_DISPLAY );
-    if (!p_eglInitialize( display, &major, &minor )) return 0;
-    TRACE( "display %p version %u.%u\n", display, major, minor );
+    EGLint count, i, pass;
 
     p_eglGetConfigs( display, NULL, 0, &count );
     configs = malloc( count * sizeof(*configs) );
@@ -1042,8 +822,73 @@ struct opengl_funcs *ANDROID_wine_get_wgl_driver( UINT version )
         if (!pass) nb_onscreen_formats = nb_pixel_formats;
     }
 
-    init_extensions();
-    return &egl_funcs;
+    *onscreen_count = nb_onscreen_formats;
+    return nb_pixel_formats;
+}
+
+static const struct opengl_driver_funcs android_driver_funcs =
+{
+    .p_init_pixel_formats = android_init_pixel_formats,
+    .p_describe_pixel_format = android_describe_pixel_format,
+    .p_init_wgl_extensions = android_init_wgl_extensions,
+    .p_set_pixel_format = android_set_pixel_format,
+    .p_context_create = android_context_create,
+    .p_context_destroy = android_context_destroy,
+    .p_context_copy = android_context_copy,
+    .p_context_share = android_context_share,
+    .p_context_make_current = android_context_make_current,
+};
+
+/**********************************************************************
+ *           ANDROID_OpenGLInit
+ */
+UINT ANDROID_OpenGLInit( UINT version, struct opengl_funcs **funcs, const struct opengl_driver_funcs **driver_funcs )
+{
+    EGLint major, minor;
+
+    if (version != WINE_OPENGL_DRIVER_VERSION)
+    {
+        ERR( "version mismatch, opengl32 wants %u but driver has %u\n", version, WINE_OPENGL_DRIVER_VERSION );
+        return STATUS_INVALID_PARAMETER;
+    }
+    if (!(egl_handle = dlopen( SONAME_LIBEGL, RTLD_NOW|RTLD_GLOBAL )))
+    {
+        ERR( "failed to load %s: %s\n", SONAME_LIBEGL, dlerror() );
+        return STATUS_NOT_SUPPORTED;
+    }
+    if (!(opengl_handle = dlopen( SONAME_LIBGLESV2, RTLD_NOW|RTLD_GLOBAL )))
+    {
+        ERR( "failed to load %s: %s\n", SONAME_LIBGLESV2, dlerror() );
+        return STATUS_NOT_SUPPORTED;
+    }
+
+#define LOAD_FUNCPTR(func) do { \
+        if (!(p_##func = dlsym( egl_handle, #func ))) \
+        { ERR( "can't find symbol %s\n", #func); return FALSE; }    \
+    } while(0)
+    LOAD_FUNCPTR( eglCreateContext );
+    LOAD_FUNCPTR( eglCreateWindowSurface );
+    LOAD_FUNCPTR( eglCreatePbufferSurface );
+    LOAD_FUNCPTR( eglDestroyContext );
+    LOAD_FUNCPTR( eglDestroySurface );
+    LOAD_FUNCPTR( eglGetConfigAttrib );
+    LOAD_FUNCPTR( eglGetConfigs );
+    LOAD_FUNCPTR( eglGetDisplay );
+    LOAD_FUNCPTR( eglGetProcAddress );
+    LOAD_FUNCPTR( eglInitialize );
+    LOAD_FUNCPTR( eglMakeCurrent );
+    LOAD_FUNCPTR( eglSwapBuffers );
+    LOAD_FUNCPTR( eglSwapInterval );
+#undef LOAD_FUNCPTR
+
+    display = p_eglGetDisplay( EGL_DEFAULT_DISPLAY );
+    if (!p_eglInitialize( display, &major, &minor )) return 0;
+    TRACE( "display %p version %u.%u\n", display, major, minor );
+
+    init_opengl_funcs();
+    *funcs = &egl_funcs;
+    *driver_funcs = &android_driver_funcs;
+    return STATUS_SUCCESS;
 }
 
 
@@ -1062,16 +907,8 @@ ALL_GL_FUNCS
 
 static struct opengl_funcs egl_funcs =
 {
-    .p_wglCopyContext = android_wglCopyContext,
-    .p_wglCreateContext = android_wglCreateContext,
-    .p_wglDeleteContext = android_wglDeleteContext,
-    .p_wglGetPixelFormat = android_wglGetPixelFormat,
     .p_wglGetProcAddress = android_wglGetProcAddress,
-    .p_wglMakeCurrent = android_wglMakeCurrent,
-    .p_wglSetPixelFormat = android_wglSetPixelFormat,
-    .p_wglShareLists = android_wglShareLists,
     .p_wglSwapBuffers = android_wglSwapBuffers,
-    .p_get_pixel_formats = android_get_pixel_formats,
 #define USE_GL_FUNC(name) .p_##name = (void *)glstub_##name,
     ALL_GL_FUNCS
 #undef USE_GL_FUNC
