@@ -491,6 +491,45 @@ NTSTATUS bluez_adapter_stop_discovery( void *connection, const char *adapter_pat
     return STATUS_SUCCESS;
 }
 
+NTSTATUS bluez_adapter_remove_device( void *connection, const char *adapter_path, const char *device_path )
+{
+    DBusMessage *request, *reply = NULL;
+    DBusError error;
+    NTSTATUS status;
+
+    TRACE( "(%p, %s, %s)\n", connection, debugstr_a( adapter_path ), debugstr_a( device_path ) );
+
+    request = p_dbus_message_new_method_call( BLUEZ_DEST, adapter_path, BLUEZ_INTERFACE_ADAPTER, "RemoveDevice" );
+    if (!request)
+        return STATUS_NO_MEMORY;
+
+    if (!p_dbus_message_append_args( request, DBUS_TYPE_OBJECT_PATH, &device_path, DBUS_TYPE_INVALID ))
+    {
+        p_dbus_message_unref( request );
+        return STATUS_NO_MEMORY;
+    }
+
+    p_dbus_error_init( &error );
+    status = bluez_dbus_send_and_wait_for_reply( connection, request, &reply, &error );
+    if (status)
+    {
+        p_dbus_error_free( &error );
+        return status;
+    }
+    if (!reply)
+    {
+        ERR( "Failed to remove device %s on adapter %s: %s: %s\n", debugstr_a( device_path ),
+             debugstr_a( adapter_path ), debugstr_a( error.name ), debugstr_a( error.message ) );
+        status = bluez_dbus_error_to_ntstatus( &error );
+        p_dbus_error_free( &error );
+        return status;
+    }
+    p_dbus_message_unref( reply );
+    p_dbus_error_free( &error );
+
+    return STATUS_SUCCESS;
+}
+
 NTSTATUS bluez_adapter_set_prop( void *connection, struct bluetooth_adapter_set_prop_params *params )
 {
     DBusMessage *request, *reply;
@@ -1194,6 +1233,39 @@ done:
     return ret;
 }
 
+NTSTATUS bluez_device_disconnect( void *connection, const char *device_path )
+{
+    DBusMessage *request, *reply = NULL;
+    NTSTATUS status;
+    DBusError error;
+
+    TRACE( "(%p, %s)\n", connection, debugstr_a( device_path ) );
+
+    request = p_dbus_message_new_method_call( BLUEZ_DEST, device_path, BLUEZ_INTERFACE_DEVICE, "Disconnect" );
+    if (!request)
+        return STATUS_NO_MEMORY;
+
+    p_dbus_error_init( &error );
+    status = bluez_dbus_send_and_wait_for_reply( connection, request, &reply, &error );
+    if (status)
+    {
+        p_dbus_error_free( &error );
+        return status;
+    }
+    if (!reply)
+    {
+        ERR( "Failed to disconnect device %s: %s: %s\n", debugstr_a( device_path ), debugstr_a( error.name ),
+             debugstr_a( error.message ) );
+        status = bluez_dbus_error_to_ntstatus( &error );
+        p_dbus_error_free( &error );
+        return status;
+    }
+    p_dbus_message_unref( reply );
+    p_dbus_error_free( &error );
+
+    return STATUS_SUCCESS;
+}
+
 struct bluez_watcher_event
 {
     struct list entry;
@@ -1908,6 +1980,7 @@ static NTSTATUS bluez_build_initial_device_lists( DBusMessage *reply, struct lis
                     goto done;
                 }
                 init_device->object.device.device.handle = (UINT_PTR)device_name;
+                init_device->object.device.init_entry = TRUE;
 
                 while((prop_name = bluez_next_dict_entry( &prop_iter, &variant )))
                 {
@@ -2105,6 +2178,10 @@ NTSTATUS bluez_adapter_stop_discovery( void *connection, const char *adapter_pat
 {
     return STATUS_NOT_SUPPORTED;
 }
+NTSTATUS bluez_adapter_remove_device( void *connection, const char *adapter_path, const char *device_path )
+{
+    return STATUS_NOT_SUPPORTED;
+}
 NTSTATUS bluez_auth_agent_start( void *connection, void **ctx ) { return STATUS_NOT_SUPPORTED; }
 NTSTATUS bluez_auth_agent_stop( void *connection, void *ctx ) { return STATUS_NOT_SUPPORTED; }
 NTSTATUS bluez_auth_agent_request_default( void *connection ) { return STATUS_NOT_SUPPORTED; }
@@ -2114,5 +2191,9 @@ NTSTATUS bluez_auth_agent_send_response( void *auth_agent, struct unix_name *dev
 {
     return STATUS_NOT_SUPPORTED;
 }
+NTSTATUS bluez_device_disconnect( void *connection, const char *device_path )
+{
+    return STATUS_NOT_SUPPORTED;
+}
 
 #endif /* SONAME_LIBDBUS_1 */
