@@ -208,6 +208,15 @@ static NTSTATUS bluetooth_adapter_stop_discovery( void *args )
     return bluez_adapter_stop_discovery( dbus_connection, params->adapter->str );
 }
 
+
+static NTSTATUS bluetooth_adapter_remove_device( void *args )
+{
+    struct bluetooth_adapter_remove_device_params *params = args;
+
+    if (!dbus_connection) return STATUS_NOT_SUPPORTED;
+    return bluez_adapter_remove_device( dbus_connection, params->adapter->str, params->device->str );
+}
+
 static NTSTATUS bluetooth_auth_agent_enable_incoming( void *args )
 {
     if (!dbus_connection) return STATUS_NOT_SUPPORTED;
@@ -223,6 +232,14 @@ static NTSTATUS bluetooth_auth_send_response( void *args )
                                            params->numeric_or_passkey, params->negative, params->authenticated );
 }
 
+static NTSTATUS bluetooth_device_disconnect( void *args )
+{
+    struct bluetooth_device_disconnect_params *params = args;
+
+    if (!dbus_connection) return STATUS_NOT_SUPPORTED;
+    return bluez_device_disconnect( dbus_connection, params->device->str );
+}
+
 static NTSTATUS bluetooth_get_event( void *args )
 {
     struct bluetooth_get_event_params *params = args;
@@ -240,9 +257,11 @@ const unixlib_entry_t __wine_unix_call_funcs[] = {
     bluetooth_adapter_get_unique_name,
     bluetooth_adapter_start_discovery,
     bluetooth_adapter_stop_discovery,
+    bluetooth_adapter_remove_device,
     bluetooth_adapter_free,
 
     bluetooth_device_free,
+    bluetooth_device_disconnect,
 
     bluetooth_auth_agent_enable_incoming,
     bluetooth_auth_send_response,
