@@ -121,6 +121,10 @@ static const IClassFactoryVtbl class_factory_vtbl =
 };
 
 static const GUID CLSID_GStreamerByteStreamHandler = {0x317df618, 0x5e5a, 0x468a, {0x9f, 0x15, 0xd8, 0x27, 0xa9, 0xa0, 0x81, 0x62}};
+static const GUID CLSID_wg_video_processor = {0xd527607f,0x89cb,0x4e94,{0x95,0x71,0xbc,0xfe,0x62,0x17,0x56,0x13}};
+static const GUID CLSID_wg_aac_decoder = {0xe7889a8a,0x2083,0x4844,{0x83,0x70,0x5e,0xe3,0x49,0xb1,0x45,0x03}};
+static const GUID CLSID_wg_h264_decoder = {0x1f1e273d,0x12c0,0x4b3a,{0x8e,0x9b,0x19,0x33,0xc2,0x49,0x8a,0xea}};
+static const GUID CLSID_wg_h264_encoder = {0x6c34de69,0x4670,0x46cd,{0x8c,0xb4,0x1f,0x2f,0xa1,0xdf,0xfb,0x65}};
 
 static const struct class_object
 {
@@ -129,11 +133,11 @@ static const struct class_object
 }
 class_objects[] =
 {
-    { &CLSID_VideoProcessorMFT, &video_processor_create },
+    { &CLSID_wg_video_processor, &video_processor_create },
     { &CLSID_GStreamerByteStreamHandler, &gstreamer_byte_stream_handler_create },
-    { &CLSID_MSAACDecMFT, &aac_decoder_create },
-    { &CLSID_MSH264DecoderMFT, &h264_decoder_create },
-    { &CLSID_MSH264EncoderMFT, &h264_encoder_create },
+    { &CLSID_wg_aac_decoder, &aac_decoder_create },
+    { &CLSID_wg_h264_decoder, &h264_decoder_create },
+    { &CLSID_wg_h264_encoder, &h264_encoder_create },
 };
 
 HRESULT mfplat_get_class_object(REFCLSID rclsid, REFIID riid, void **obj)
@@ -162,298 +166,6 @@ HRESULT mfplat_get_class_object(REFCLSID rclsid, REFIID riid, void **obj)
     return CLASS_E_CLASSNOTAVAILABLE;
 }
 
-HRESULT mfplat_DllRegisterServer(void)
-{
-    MFT_REGISTER_TYPE_INFO resampler_types[] =
-    {
-        {MFMediaType_Audio, MFAudioFormat_PCM},
-        {MFMediaType_Audio, MFAudioFormat_Float},
-    };
-
-    MFT_REGISTER_TYPE_INFO aac_decoder_input_types[] =
-    {
-        {MFMediaType_Audio, MFAudioFormat_AAC},
-        {MFMediaType_Audio, MFAudioFormat_RAW_AAC},
-        {MFMediaType_Audio, MFAudioFormat_ADTS},
-    };
-    MFT_REGISTER_TYPE_INFO aac_decoder_output_types[] =
-    {
-        {MFMediaType_Audio, MFAudioFormat_Float},
-        {MFMediaType_Audio, MFAudioFormat_PCM},
-    };
-
-    MFT_REGISTER_TYPE_INFO wma_decoder_input_types[] =
-    {
-        {MFMediaType_Audio, MEDIASUBTYPE_MSAUDIO1},
-        {MFMediaType_Audio, MFAudioFormat_WMAudioV8},
-        {MFMediaType_Audio, MFAudioFormat_WMAudioV9},
-        {MFMediaType_Audio, MFAudioFormat_WMAudio_Lossless},
-    };
-    MFT_REGISTER_TYPE_INFO wma_decoder_output_types[] =
-    {
-        {MFMediaType_Audio, MFAudioFormat_PCM},
-        {MFMediaType_Audio, MFAudioFormat_Float},
-    };
-
-    MFT_REGISTER_TYPE_INFO h264_decoder_input_types[] =
-    {
-        {MFMediaType_Video, MFVideoFormat_H264},
-        {MFMediaType_Video, MFVideoFormat_H264_ES},
-    };
-    MFT_REGISTER_TYPE_INFO h264_decoder_output_types[] =
-    {
-        {MFMediaType_Video, MFVideoFormat_NV12},
-        {MFMediaType_Video, MFVideoFormat_YV12},
-        {MFMediaType_Video, MFVideoFormat_IYUV},
-        {MFMediaType_Video, MFVideoFormat_I420},
-        {MFMediaType_Video, MFVideoFormat_YUY2},
-    };
-
-    MFT_REGISTER_TYPE_INFO h264_encoder_input_types[] =
-    {
-        {MFMediaType_Video, MFVideoFormat_IYUV},
-        {MFMediaType_Video, MFVideoFormat_YV12},
-        {MFMediaType_Video, MFVideoFormat_NV12},
-        {MFMediaType_Video, MFVideoFormat_YUY2},
-    };
-    MFT_REGISTER_TYPE_INFO h264_encoder_output_types[] =
-    {
-        {MFMediaType_Video, MFVideoFormat_H264},
-    };
-
-    MFT_REGISTER_TYPE_INFO video_processor_input_types[] =
-    {
-        {MFMediaType_Video, MFVideoFormat_IYUV},
-        {MFMediaType_Video, MFVideoFormat_YV12},
-        {MFMediaType_Video, MFVideoFormat_NV12},
-        {MFMediaType_Video, MFVideoFormat_YUY2},
-        {MFMediaType_Video, MFVideoFormat_ARGB32},
-        {MFMediaType_Video, MFVideoFormat_RGB32},
-        {MFMediaType_Video, MFVideoFormat_NV11},
-        {MFMediaType_Video, MFVideoFormat_AYUV},
-        {MFMediaType_Video, MFVideoFormat_UYVY},
-        {MFMediaType_Video, MEDIASUBTYPE_P208},
-        {MFMediaType_Video, MFVideoFormat_RGB24},
-        {MFMediaType_Video, MFVideoFormat_RGB555},
-        {MFMediaType_Video, MFVideoFormat_RGB565},
-        {MFMediaType_Video, MFVideoFormat_RGB8},
-        {MFMediaType_Video, MFVideoFormat_I420},
-        {MFMediaType_Video, MFVideoFormat_Y216},
-        {MFMediaType_Video, MFVideoFormat_v410},
-        {MFMediaType_Video, MFVideoFormat_Y41P},
-        {MFMediaType_Video, MFVideoFormat_Y41T},
-        {MFMediaType_Video, MFVideoFormat_Y42T},
-        {MFMediaType_Video, MFVideoFormat_YVYU},
-        {MFMediaType_Video, MFVideoFormat_420O},
-    };
-    MFT_REGISTER_TYPE_INFO video_processor_output_types[] =
-    {
-        {MFMediaType_Video, MFVideoFormat_IYUV},
-        {MFMediaType_Video, MFVideoFormat_YV12},
-        {MFMediaType_Video, MFVideoFormat_NV12},
-        {MFMediaType_Video, MFVideoFormat_YUY2},
-        {MFMediaType_Video, MFVideoFormat_ARGB32},
-        {MFMediaType_Video, MFVideoFormat_RGB32},
-        {MFMediaType_Video, MFVideoFormat_NV11},
-        {MFMediaType_Video, MFVideoFormat_AYUV},
-        {MFMediaType_Video, MFVideoFormat_UYVY},
-        {MFMediaType_Video, MEDIASUBTYPE_P208},
-        {MFMediaType_Video, MFVideoFormat_RGB24},
-        {MFMediaType_Video, MFVideoFormat_RGB555},
-        {MFMediaType_Video, MFVideoFormat_RGB565},
-        {MFMediaType_Video, MFVideoFormat_RGB8},
-        {MFMediaType_Video, MFVideoFormat_I420},
-        {MFMediaType_Video, MFVideoFormat_Y216},
-        {MFMediaType_Video, MFVideoFormat_v410},
-        {MFMediaType_Video, MFVideoFormat_Y41P},
-        {MFMediaType_Video, MFVideoFormat_Y41T},
-        {MFMediaType_Video, MFVideoFormat_Y42T},
-        {MFMediaType_Video, MFVideoFormat_YVYU},
-    };
-
-    MFT_REGISTER_TYPE_INFO wmv_decoder_input_types[] =
-    {
-        {MFMediaType_Video, MFVideoFormat_WMV1},
-        {MFMediaType_Video, MFVideoFormat_WMV2},
-        {MFMediaType_Video, MFVideoFormat_WMV3},
-        {MFMediaType_Video, MEDIASUBTYPE_WMVP},
-        {MFMediaType_Video, MEDIASUBTYPE_WVP2},
-        {MFMediaType_Video, MEDIASUBTYPE_WMVR},
-        {MFMediaType_Video, MEDIASUBTYPE_WMVA},
-        {MFMediaType_Video, MFVideoFormat_WVC1},
-        {MFMediaType_Video, MFVideoFormat_VC1S},
-    };
-    MFT_REGISTER_TYPE_INFO wmv_decoder_output_types[] =
-    {
-        {MFMediaType_Video, MFVideoFormat_YV12},
-        {MFMediaType_Video, MFVideoFormat_YUY2},
-        {MFMediaType_Video, MFVideoFormat_UYVY},
-        {MFMediaType_Video, MFVideoFormat_YVYU},
-        {MFMediaType_Video, MFVideoFormat_NV11},
-        {MFMediaType_Video, MFVideoFormat_NV12},
-        {MFMediaType_Video, DMOVideoFormat_RGB32},
-        {MFMediaType_Video, DMOVideoFormat_RGB24},
-        {MFMediaType_Video, DMOVideoFormat_RGB565},
-        {MFMediaType_Video, DMOVideoFormat_RGB555},
-        {MFMediaType_Video, DMOVideoFormat_RGB8},
-    };
-
-    MFT_REGISTER_TYPE_INFO color_convert_input_types[] =
-    {
-        {MFMediaType_Video, MFVideoFormat_YV12},
-        {MFMediaType_Video, MFVideoFormat_YUY2},
-        {MFMediaType_Video, MFVideoFormat_UYVY},
-        {MFMediaType_Video, MFVideoFormat_AYUV},
-        {MFMediaType_Video, MFVideoFormat_NV12},
-        {MFMediaType_Video, DMOVideoFormat_RGB32},
-        {MFMediaType_Video, DMOVideoFormat_RGB565},
-        {MFMediaType_Video, MFVideoFormat_I420},
-        {MFMediaType_Video, MFVideoFormat_IYUV},
-        {MFMediaType_Video, MFVideoFormat_YVYU},
-        {MFMediaType_Video, DMOVideoFormat_RGB24},
-        {MFMediaType_Video, DMOVideoFormat_RGB555},
-        {MFMediaType_Video, DMOVideoFormat_RGB8},
-        {MFMediaType_Video, MEDIASUBTYPE_V216},
-        {MFMediaType_Video, MEDIASUBTYPE_V410},
-        {MFMediaType_Video, MFVideoFormat_NV11},
-        {MFMediaType_Video, MFVideoFormat_Y41P},
-        {MFMediaType_Video, MFVideoFormat_Y41T},
-        {MFMediaType_Video, MFVideoFormat_Y42T},
-        {MFMediaType_Video, MFVideoFormat_YVU9},
-    };
-    MFT_REGISTER_TYPE_INFO color_convert_output_types[] =
-    {
-        {MFMediaType_Video, MFVideoFormat_YV12},
-        {MFMediaType_Video, MFVideoFormat_YUY2},
-        {MFMediaType_Video, MFVideoFormat_UYVY},
-        {MFMediaType_Video, MFVideoFormat_AYUV},
-        {MFMediaType_Video, MFVideoFormat_NV12},
-        {MFMediaType_Video, DMOVideoFormat_RGB32},
-        {MFMediaType_Video, DMOVideoFormat_RGB565},
-        {MFMediaType_Video, MFVideoFormat_I420},
-        {MFMediaType_Video, MFVideoFormat_IYUV},
-        {MFMediaType_Video, MFVideoFormat_YVYU},
-        {MFMediaType_Video, DMOVideoFormat_RGB24},
-        {MFMediaType_Video, DMOVideoFormat_RGB555},
-        {MFMediaType_Video, DMOVideoFormat_RGB8},
-        {MFMediaType_Video, MEDIASUBTYPE_V216},
-        {MFMediaType_Video, MEDIASUBTYPE_V410},
-        {MFMediaType_Video, MFVideoFormat_NV11},
-    };
-
-    struct mft
-    {
-        GUID clsid;
-        GUID category;
-        WCHAR name[MAX_PATH];
-        UINT32 flags;
-        UINT32 input_types_count;
-        MFT_REGISTER_TYPE_INFO *input_types;
-        UINT32 output_types_count;
-        MFT_REGISTER_TYPE_INFO *output_types;
-    }
-    mfts[] =
-    {
-        {
-            CLSID_MSAACDecMFT,
-            MFT_CATEGORY_AUDIO_DECODER,
-            L"Microsoft AAC Audio Decoder MFT",
-            MFT_ENUM_FLAG_SYNCMFT,
-            ARRAY_SIZE(aac_decoder_input_types),
-            aac_decoder_input_types,
-            ARRAY_SIZE(aac_decoder_output_types),
-            aac_decoder_output_types,
-        },
-        {
-            CLSID_WMADecMediaObject,
-            MFT_CATEGORY_AUDIO_DECODER,
-            L"WMAudio Decoder MFT",
-            MFT_ENUM_FLAG_SYNCMFT,
-            ARRAY_SIZE(wma_decoder_input_types),
-            wma_decoder_input_types,
-            ARRAY_SIZE(wma_decoder_output_types),
-            wma_decoder_output_types,
-        },
-        {
-            CLSID_MSH264DecoderMFT,
-            MFT_CATEGORY_VIDEO_DECODER,
-            L"Microsoft H264 Video Decoder MFT",
-            MFT_ENUM_FLAG_SYNCMFT,
-            ARRAY_SIZE(h264_decoder_input_types),
-            h264_decoder_input_types,
-            ARRAY_SIZE(h264_decoder_output_types),
-            h264_decoder_output_types,
-        },
-        {
-            CLSID_MSH264EncoderMFT,
-            MFT_CATEGORY_VIDEO_ENCODER,
-            L"H264 Encoder MFT",
-            MFT_ENUM_FLAG_SYNCMFT,
-            ARRAY_SIZE(h264_encoder_input_types),
-            h264_encoder_input_types,
-            ARRAY_SIZE(h264_encoder_output_types),
-            h264_encoder_output_types,
-        },
-        {
-            CLSID_WMVDecoderMFT,
-            MFT_CATEGORY_VIDEO_DECODER,
-            L"WMVideo Decoder MFT",
-            MFT_ENUM_FLAG_SYNCMFT,
-            ARRAY_SIZE(wmv_decoder_input_types),
-            wmv_decoder_input_types,
-            ARRAY_SIZE(wmv_decoder_output_types),
-            wmv_decoder_output_types,
-        },
-        {
-            CLSID_VideoProcessorMFT,
-            MFT_CATEGORY_VIDEO_PROCESSOR,
-            L"Microsoft Video Processor MFT",
-            MFT_ENUM_FLAG_SYNCMFT,
-            ARRAY_SIZE(video_processor_input_types),
-            video_processor_input_types,
-            ARRAY_SIZE(video_processor_output_types),
-            video_processor_output_types,
-        },
-        {
-            CLSID_CResamplerMediaObject,
-            MFT_CATEGORY_AUDIO_EFFECT,
-            L"Resampler MFT",
-            MFT_ENUM_FLAG_SYNCMFT,
-            ARRAY_SIZE(resampler_types),
-            resampler_types,
-            ARRAY_SIZE(resampler_types),
-            resampler_types,
-        },
-        {
-            CLSID_CColorConvertDMO,
-            MFT_CATEGORY_VIDEO_EFFECT,
-            L"Color Converter MFT",
-            MFT_ENUM_FLAG_SYNCMFT,
-            ARRAY_SIZE(color_convert_input_types),
-            color_convert_input_types,
-            ARRAY_SIZE(color_convert_output_types),
-            color_convert_output_types,
-        },
-    };
-
-    unsigned int i;
-    HRESULT hr;
-
-    for (i = 0; i < ARRAY_SIZE(mfts); i++)
-    {
-        hr = MFTRegister(mfts[i].clsid, mfts[i].category, mfts[i].name, mfts[i].flags, mfts[i].input_types_count,
-                    mfts[i].input_types, mfts[i].output_types_count, mfts[i].output_types, NULL);
-
-        if (FAILED(hr))
-        {
-            FIXME("Failed to register MFT, hr %#lx.\n", hr);
-            return hr;
-        }
-    }
-
-    return S_OK;
-}
-
 static const struct
 {
     const GUID *subtype;
