@@ -30,6 +30,8 @@ WINE_DECLARE_DEBUG_CHANNEL(winediag);
 
 extern GUID MFVideoFormat_ABGR32;
 
+static const GUID MF_XVP_PLAYBACK_MODE = { 0x3c5d293f, 0xad67, 0x4e29, { 0xaf, 0x12, 0xcf, 0x3e, 0x23, 0x8a, 0xcc, 0xe9 } };
+
 static const GUID *const input_types[] =
 {
     &MFVideoFormat_IYUV,
@@ -91,34 +93,69 @@ struct video_processor
     IMFVideoSampleAllocatorEx *allocator;
 };
 
-static HRESULT normalize_stride(IMFMediaType *media_type, BOOL bottom_up, IMFMediaType **ret)
+static void update_video_aperture(MFVideoInfo *input_info, MFVideoInfo *output_info)
 {
-    MFVIDEOFORMAT *format;
-    LONG stride;
+    RECT input_rect, output_rect;
+
+    get_mf_video_content_rect(input_info, &input_rect);
+    get_mf_video_content_rect(output_info, &output_rect);
+
+    if (!EqualRect(&input_rect, &output_rect))
+    {
+        FIXME("Mismatched content size %s vs %s\n", wine_dbgstr_rect(&input_rect),
+                wine_dbgstr_rect(&output_rect));
+    }
+
+    input_info->MinimumDisplayAperture.OffsetX.value = input_rect.left;
+    input_info->MinimumDisplayAperture.OffsetY.value = input_rect.top;
+    input_info->MinimumDisplayAperture.Area.cx = input_rect.right - input_rect.left;
+    input_info->MinimumDisplayAperture.Area.cy = input_rect.bottom - input_rect.top;
+    output_info->MinimumDisplayAperture = input_info->MinimumDisplayAperture;
+}
+
+static HRESULT normalize_media_types(BOOL bottom_up, IMFMediaType **input_type, IMFMediaType **output_type)
+{
+    MFVIDEOFORMAT *input_format, *output_format;
+    BOOL normalize_input, normalize_output;
     UINT32 size;
     HRESULT hr;
 
-    if (SUCCEEDED(hr = IMFMediaType_GetUINT32(media_type, &MF_MT_DEFAULT_STRIDE, (UINT32 *)&stride)))
+    normalize_input = FAILED(IMFMediaType_GetItem(*input_type, &MF_MT_DEFAULT_STRIDE, NULL));
+    normalize_output = FAILED(IMFMediaType_GetItem(*output_type, &MF_MT_DEFAULT_STRIDE, NULL));
+
+    if (FAILED(hr = MFCreateMFVideoFormatFromMFMediaType(*input_type, &input_format, &size)))
+        return hr;
+    if (FAILED(hr = MFCreateMFVideoFormatFromMFMediaType(*output_type, &output_format, &size)))
     {
-        *ret = media_type;
-        IMFMediaType_AddRef(media_type);
+        CoTaskMemFree(input_format);
         return hr;
     }
 
-    if (SUCCEEDED(hr = MFCreateMFVideoFormatFromMFMediaType(media_type, &format, &size)))
+    if (bottom_up && normalize_input)
+        input_format->videoInfo.VideoFlags |= MFVideoFlag_BottomUpLinearRep;
+    if (bottom_up && normalize_output)
+        output_format->videoInfo.VideoFlags |= MFVideoFlag_BottomUpLinearRep;
+
+    update_video_aperture(&input_format->videoInfo, &output_format->videoInfo);
+
+    if (FAILED(hr = MFCreateVideoMediaType(input_format, (IMFVideoMediaType **)input_type)))
+        goto done;
+    if (FAILED(hr = MFCreateVideoMediaType(output_format, (IMFVideoMediaType **)output_type)))
     {
-        if (bottom_up) format->videoInfo.VideoFlags |= MFVideoFlag_BottomUpLinearRep;
-        hr = MFCreateVideoMediaType(format, (IMFVideoMediaType **)ret);
-        CoTaskMemFree(format);
+        IMFMediaType_Release(*input_type);
+        *input_type = NULL;
     }
 
+done:
+    CoTaskMemFree(input_format);
+    CoTaskMemFree(output_format);
     return hr;
 }
 
 static HRESULT try_create_wg_transform(struct video_processor *impl)
 {
     BOOL bottom_up = !impl->device_manager; /* when not D3D-enabled, the transform outputs bottom up RGB buffers */
-    IMFMediaType *input_type, *output_type;
+    IMFMediaType *input_type = impl->input_type, *output_type = impl->output_type;
     struct wg_transform_attrs attrs = {0};
     HRESULT hr;
 
@@ -128,13 +165,8 @@ static HRESULT try_create_wg_transform(struct video_processor *impl)
         impl->wg_transform = 0;
     }
 
-    if (FAILED(hr = normalize_stride(impl->input_type, bottom_up, &input_type)))
+    if (FAILED(hr = normalize_media_types(bottom_up, &input_type, &output_type)))
         return hr;
-    if (FAILED(hr = normalize_stride(impl->output_type, bottom_up, &output_type)))
-    {
-        IMFMediaType_Release(input_type);
-        return hr;
-    }
     hr = wg_transform_create_mf(input_type, output_type, &attrs, &impl->wg_transform);
     IMFMediaType_Release(output_type);
     IMFMediaType_Release(input_type);
@@ -666,6 +698,7 @@ static HRESULT WINAPI video_processor_ProcessOutput(IMFTransform *iface, DWORD f
     MFT_OUTPUT_STREAM_INFO info;
     IMFSample *output_sample;
     HRESULT hr;
+    BOOL playback_mode, provide_samples;
 
     TRACE("iface %p, flags %#lx, count %lu, samples %p, status %p.\n", iface, flags, count, samples, status);
 
@@ -679,7 +712,12 @@ static HRESULT WINAPI video_processor_ProcessOutput(IMFTransform *iface, DWORD f
     if (FAILED(hr = IMFTransform_GetOutputStreamInfo(iface, 0, &info)))
         return hr;
 
-    if (impl->output_info.dwFlags & MFT_OUTPUT_STREAM_PROVIDES_SAMPLES)
+    if (FAILED(IMFAttributes_GetUINT32(impl->attributes, &MF_XVP_PLAYBACK_MODE, (UINT32 *) &playback_mode)))
+        playback_mode = FALSE;
+
+    provide_samples = (impl->output_info.dwFlags & MFT_OUTPUT_STREAM_PROVIDES_SAMPLES) && !playback_mode;
+
+    if (provide_samples)
     {
         if (FAILED(hr = video_processor_init_allocator(impl)))
             return hr;
@@ -697,7 +735,7 @@ static HRESULT WINAPI video_processor_ProcessOutput(IMFTransform *iface, DWORD f
         goto done;
     wg_sample_queue_flush(impl->wg_sample_queue, false);
 
-    if (impl->output_info.dwFlags & MFT_OUTPUT_STREAM_PROVIDES_SAMPLES)
+    if (provide_samples)
     {
         samples->pSample = output_sample;
         IMFSample_AddRef(output_sample);
