@@ -176,7 +176,9 @@ static void macdrv_im_set_text(const macdrv_event *event)
     }
 
     if (event->im_set_text.complete) post_ime_update(hwnd, -1, NULL, text);
-    else post_ime_update(hwnd, event->im_set_text.cursor_pos, text, NULL);
+    else post_ime_update(hwnd,
+                         MAKELONG(event->im_set_text.cursor_begin, event->im_set_text.cursor_end),
+                         text, NULL);
 
     free(text);
 }
@@ -223,60 +225,60 @@ static uint32_t dropeffect_to_drag_operation(DWORD effect, uint32_t ops)
 
 
 /**************************************************************************
- *              query_drag_drop
+ *              query_drag_drop_drop
  */
-static BOOL query_drag_drop(macdrv_query *query)
+static BOOL query_drag_drop_drop(macdrv_query *query)
 {
     HWND hwnd = macdrv_get_window_hwnd(query->window);
-    struct macdrv_win_data *data = get_win_data(hwnd);
-    struct dnd_query_drop_params params = {.dispatch = {.callback = dnd_query_drop_callback}};
-    void *ret_ptr;
-    ULONG ret_len;
+    struct macdrv_win_data *data;
 
-    if (!data)
+    if (!(data = get_win_data(hwnd)))
     {
         WARN("no win_data for win %p/%p\n", hwnd, query->window);
         return FALSE;
     }
 
-    params.hwnd = HandleToUlong(hwnd);
-    params.effect = drag_operations_to_dropeffects(query->drag_drop.op);
-    params.x = query->drag_drop.x + data->rects.visible.left;
-    params.y = query->drag_drop.y + data->rects.visible.top;
-    params.handle = (UINT_PTR)query->drag_drop.pasteboard;
     release_win_data(data);
-    if (KeUserDispatchCallback(&params.dispatch, sizeof(params), &ret_ptr, &ret_len))
-        return FALSE;
-    return *(BOOL *)ret_ptr;
+
+    NtUserMessageCall(hwnd, WINE_DRAG_DROP_DROP, 0, 0, NULL, NtUserDragDropCall, FALSE);
+    return TRUE;
 }
 
 /**************************************************************************
- *              query_drag_exited
+ *              query_drag_drop_enter
  */
-static BOOL query_drag_exited(macdrv_query *query)
+static BOOL query_drag_drop_enter(macdrv_query *query)
 {
-    struct dnd_query_exited_params params = {.dispatch = {.callback = dnd_query_exited_callback}};
-    void *ret_ptr;
-    ULONG ret_len;
+    CFTypeRef pasteboard = query->drag_drop.pasteboard;
+    struct format_entry *entries;
+    UINT entries_size;
 
-    params.hwnd = HandleToUlong(macdrv_get_window_hwnd(query->window));
-    if (KeUserDispatchCallback(&params.dispatch, sizeof(params), &ret_ptr, &ret_len))
-        return FALSE;
-    return *(BOOL *)ret_ptr;
+    if (!(entries = get_format_entries(pasteboard, &entries_size))) return FALSE;
+    NtUserMessageCall(0, WINE_DRAG_DROP_ENTER, entries_size, (LPARAM)entries, NULL, NtUserDragDropCall, FALSE);
+    free(entries);
+
+    return TRUE;
+}
+
+/**************************************************************************
+ *              query_drag_drop_leave
+ */
+static BOOL query_drag_drop_leave(macdrv_query *query)
+{
+    NtUserMessageCall(0, WINE_DRAG_DROP_LEAVE, 0, 0, NULL, NtUserDragDropCall, FALSE);
+    return TRUE;
 }
 
 
 /**************************************************************************
- *              query_drag_operation
+ *              query_drag_drop_drag
  */
-static BOOL query_drag_operation(macdrv_query *query)
+static BOOL query_drag_drop_drag(macdrv_query *query)
 {
-    struct dnd_query_drag_params params = {.dispatch = {.callback = dnd_query_drag_callback}};
     HWND hwnd = macdrv_get_window_hwnd(query->window);
     struct macdrv_win_data *data = get_win_data(hwnd);
-    void *ret_ptr;
-    ULONG ret_len;
     DWORD effect;
+    POINT point;
 
     if (!data)
     {
@@ -284,20 +286,15 @@ static BOOL query_drag_operation(macdrv_query *query)
         return FALSE;
     }
 
-    params.hwnd = HandleToUlong(hwnd);
-    params.effect = drag_operations_to_dropeffects(query->drag_operation.offered_ops);
-    params.x = query->drag_operation.x + data->rects.visible.left;
-    params.y = query->drag_operation.y + data->rects.visible.top;
-    params.handle = (UINT_PTR)query->drag_operation.pasteboard;
+    effect = drag_operations_to_dropeffects(query->drag_drop.ops);
+    point.x = query->drag_drop.x + data->rects.visible.left;
+    point.y = query->drag_drop.y + data->rects.visible.top;
     release_win_data(data);
 
-    if (KeUserDispatchCallback(&params.dispatch, sizeof(params), &ret_ptr, &ret_len))
-        return FALSE;
-    effect = *(DWORD *)ret_ptr;
+    effect = NtUserMessageCall(hwnd, WINE_DRAG_DROP_DRAG, MAKELONG(point.x, point.y), effect, NULL, NtUserDragDropCall, FALSE);
     if (!effect) return FALSE;
 
-    query->drag_operation.accepted_op = dropeffect_to_drag_operation(effect,
-                                                                     query->drag_operation.offered_ops);
+    query->drag_drop.ops = dropeffect_to_drag_operation(effect, query->drag_drop.ops);
     return TRUE;
 }
 
@@ -360,17 +357,21 @@ static void macdrv_query_event(HWND hwnd, const macdrv_event *event)
 
     switch (query->type)
     {
-        case QUERY_DRAG_DROP:
-            TRACE("QUERY_DRAG_DROP\n");
-            success = query_drag_drop(query);
+        case QUERY_DRAG_DROP_ENTER:
+            TRACE("QUERY_DRAG_DROP_ENTER\n");
+            success = query_drag_drop_enter(query);
             break;
-        case QUERY_DRAG_EXITED:
-            TRACE("QUERY_DRAG_EXITED\n");
-            success = query_drag_exited(query);
+        case QUERY_DRAG_DROP_LEAVE:
+            TRACE("QUERY_DRAG_DROP_LEAVE\n");
+            success = query_drag_drop_leave(query);
             break;
-        case QUERY_DRAG_OPERATION:
-            TRACE("QUERY_DRAG_OPERATION\n");
-            success = query_drag_operation(query);
+        case QUERY_DRAG_DROP_DRAG:
+            TRACE("QUERY_DRAG_DROP_DRAG\n");
+            success = query_drag_drop_drag(query);
+            break;
+        case QUERY_DRAG_DROP_DROP:
+            TRACE("QUERY_DRAG_DROP_DROP\n");
+            success = query_drag_drop_drop(query);
             break;
         case QUERY_IME_CHAR_RECT:
             TRACE("QUERY_IME_CHAR_RECT\n");
