@@ -2135,6 +2135,17 @@ NTSTATUS WINAPI wow64_NtGdiLineTo( UINT *args )
     return NtGdiLineTo( hdc, x, y );
 }
 
+NTSTATUS WINAPI wow64_NtGdiMakeFontDir( UINT *args )
+{
+    DWORD embed = get_ulong( &args );
+    BYTE *buffer = get_ptr( &args );
+    UINT size = get_ulong( &args );
+    WCHAR *path = get_ptr( &args );
+    UINT len = get_ulong( &args );
+
+    return NtGdiMakeFontDir( embed, buffer, size, path, len );
+}
+
 NTSTATUS WINAPI wow64_NtGdiMaskBlt( UINT *args )
 {
     HDC hdc = get_handle( &args );
@@ -2818,13 +2829,3 @@ NTSTATUS WINAPI wow64___wine_get_icm_profile( UINT *args )
 
     return __wine_get_icm_profile( hdc, allow_default, size, filename );
 }
-
-NTSTATUS WINAPI wow64___wine_get_file_outline_text_metric( UINT *args )
-{
-    const WCHAR *path = get_ptr( &args );
-    TEXTMETRICW *otm = get_ptr( &args );
-    UINT *em_square = get_ptr( &args );
-    WCHAR *face_name = get_ptr( &args );
-
-    return __wine_get_file_outline_text_metric( path, otm, em_square, face_name );
-}
