@@ -113,8 +113,19 @@ struct opengl_funcs
 /* interface between win32u and the user drivers */
 struct opengl_driver_funcs
 {
+    UINT (*p_init_pixel_formats)(UINT*);
+    BOOL (*p_describe_pixel_format)(int,struct wgl_pixel_format*);
     const char *(*p_init_wgl_extensions)(void);
     BOOL (*p_set_pixel_format)(HWND,int,int,BOOL);
+    BOOL (*p_context_create)(HDC,int,void*,const int*,void**);
+    BOOL (*p_context_destroy)(void*);
+    BOOL (*p_context_copy)(void*,void*,UINT);
+    BOOL (*p_context_share)(void*,void*);
+    BOOL (*p_context_make_current)(HDC,HDC,void*);
+    BOOL (*p_pbuffer_create)(HDC,int,BOOL,GLenum,GLenum,GLint,GLsizei*,GLsizei*,void **);
+    BOOL (*p_pbuffer_destroy)(HDC,void*);
+    BOOL (*p_pbuffer_updated)(HDC,void*,GLenum,GLint);
+    UINT (*p_pbuffer_bind)(HDC,void*,GLenum);
 };
 
 #endif /* WINE_UNIX_LIB */
